#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2025, Shreyas Srish (@shrsr) <ssrish@cisco.com>

# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type

ANSIBLE_METADATA = {"metadata_version": "1.1", "status": ["preview"], "supported_by": "community"}

DOCUMENTATION = r"""
---
module: ndo_l3out_interface_group_policy
short_description: Manage L3Out Interface Group Policy on Cisco Nexus Dashboard Orchestrator (NDO).
description:
- Manage L3Out Interface Group Policy on Cisco Nexus Dashboard Orchestrator (NDO).
- This module is only supported on ND v3.1 (NDO v4.3) and later.
author:
- Shreyas Srish (@shrsr)
options:
  template:
    description:
    - The name of the template.
    - The template must be an L3Out template.
    - This parameter or O(template_id) is required.
    type: str
    aliases: [ l3out_template ]
  template_id:
    description:
    - The ID of the L3Out template.
    - This parameter or O(template) is required.
    type: str
    aliases: [ l3out_template_id ]
  l3out:
    description:
    - The name of the L3Out.
    - This parameter or O(l3out_uuid) is required.
    type: str
    aliases: [ l3out_name ]
  l3out_uuid:
    description:
    - The UUID of the L3Out.
    - This parameter or O(l3out) is required.
    type: str
  description:
    description:
    - The description of the L3Out Interface Group Policy.
    type: str
  name:
    description:
    - The name of the L3Out Interface Group Policy.
    type: str
    aliases: [ l3out_interface_group_policy ]
  interface_routing_policy:
    description:
    - The configuration of the Interface Routing Policy.
    - Providing an empty dictionary will remove the O(interface_routing_policy={}) from L3Out Interface Group Policy.
    type: dict
    suboptions:
      name:
        description:
        - The name of the L3Out Interface Routing Policy.
        type: str
      template:
        description:
        - The template associated with the L3Out Interface Routing Policy.
        - This parameter or O(interface_routing_policy.template_id) is required.
        type: str
      template_id:
        description:
        - The template ID associated with the L3Out Interface Routing Policy.
        - This parameter or O(interface_routing_policy.template) is required.
        type: str
  interface_routing_policy_uuid:
    description:
    - The UUID of the L3Out Interface Routing Policy.
    - Providing an empty string will remove the O(interface_routing_policy_uuid="") from L3Out Interface Group Policy.
    type: str
  bfd:
    description:
    - The Bidirectional Forwarding Detection (BFD) configuration.
    - Providing an empty dictionary will remove the O(bfd={}) from the L3Out Interface Group Policy.
    type: dict
    suboptions:
      enable:
        description:
        - Whether BFD is enabled.
        type: bool
      authentication:
        description:
        - Whether BFD authentication is enabled.
        type: bool
      key_id:
        description:
        - The BFD key ID.
        type: int
      key:
        description:
        - The BFD key.
        type: str
  bfd_multi_hop:
    description:
    - The Bidirectional Forwarding Detection (BFD) multi-hop configuration.
    - Providing an empty dictionary will remove the O(bfd_multi_hop={}) from the L3Out Interface Group Policy.
    type: dict
    suboptions:
      enable:
        description:
        - Whether BFD multi-hop is enabled.
        type: bool
      authentication:
        description:
        - Whether BFD multi-hop authentication is enabled.
        type: bool
      key_id:
        description:
        - The BFD multi-hop key ID.
        type: int
      key:
        description:
        - The BFD multi-hop key.
        type: str
  ospf:
    description:
    - The Open Shortest Path First (OSPF) configuration.
    - Providing an empty dictionary will remove the O(ospf={}) from the L3Out Interface Group Policy.
    type: dict
    suboptions:
      enable:
        description:
        - Whether OSPF is enabled.
        type: bool
      authentication_type:
        description:
        - The type of OSPF authentication.
        type: str
        choices: [ simple, md5, none ]
      key_id:
        description:
        - The OSPF key ID.
        type: int
      key:
        description:
        - The OSPF key.
        type: str
  qos_priority:
    description:
    - The QoS priority level.
    type: str
    choices: [ level1, level2, level3, level4, level5, level6, unspecified ]
  custom_qos_policy:
    description:
    - The Custom QoS Policy configuration.
    - Providing an empty dictionary will remove the O(custom_qos_policy={}) from L3Out Interface Group Policy.
    type: dict
    suboptions:
      name:
        description:
        - The name of the Custom QoS Policy.
        type: str
      template:
        description:
        - The template associated with the Custom QoS Policy.
        - This parameter or O(custom_qos_policy.template_id) is required.
        type: str
      template_id:
        description:
        - The template ID associated with the the Custom QoS Policy.
        - This parameter or O(custom_qos_policy.template) is required.
        type: str
  custom_qos_policy_uuid:
    description:
    - The UUID of the Custom QoS Policy.
    - Providing an empty string will remove the O(custom_qos_policy_uuid="") from L3Out Interface Group Policy.
    type: str
  state:
    description:
    - Determines the desired state of the resource.
    - Use C(absent) to remove the resource.
    - Use C(query) to list the resource.
    - Use C(present) to create or update the resource.
    type: str
    choices: [ absent, query, present ]
    default: query
notes:
- The O(template) must exist before using this module in your playbook.
  The M(cisco.mso.ndo_template) module can be used for this.
- The O(l3out) must exist before using this module in your playbook.
  The M(cisco.mso.ndo_l3out_template) module can be used for this.
- The O(interface_routing_policy) must exist before using it in the module.
  Use M(cisco.mso.ndo_l3out_interface_routing_policy) to create the L3Out Node Routing Policy.
- The O(custom_qos_policy) must exist before using it in the module.
  Use M(cisco.mso.ndo_tenant_custom_qos_policy) to create the Custom QoS Policy.
seealso:
- module: cisco.mso.ndo_template
- module: cisco.mso.ndo_l3out_template
- module: cisco.mso.ndo_l3out_interface_routing_policy
- module: cisco.mso.ndo_tenant_custom_qos_policy
extends_documentation_fragment: cisco.mso.modules
"""

EXAMPLES = r"""
- name: Create an L3Out interface group policy
  cisco.mso.ndo_l3out_interface_group_policy:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: l3out_template
    l3out: l3out
    name: interface_group_policy_1
    description: Test description
    interface_routing_policy:
      name: ansible_l3out_interface_routing_policy
      template: ansible_tenant_template
    bfd:
      enable: true
      authentication: true
      key_id: 1
      key: TestKey
    bfd_multi_hop:
      enable: true
      authentication: true
      key_id: 1
      key: TestKey
    ospf:
      enable: true
      authentication_type: simple
      key_id: 1
      key: TestKey
    custom_qos_policy:
      template: ansible_tenant_template
      name: ansible_custom_qos_policy
    qos_priority: level1
    state: present

- name: Query an existing L3Out interface group policy with name
  cisco.mso.ndo_l3out_interface_group_policy:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: l3out_template
    l3out: l3out
    name: interface_group_policy_1
    state: query
  register: query_with_name

- name: Query all existing L3Out interface group policies under l3out
  cisco.mso.ndo_l3out_interface_group_policy:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: l3out_template
    l3out: l3out
    state: query

- name: Delete an existing L3Out interface group policy with name
  cisco.mso.ndo_l3out_interface_group_policy:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: l3out_template
    l3out: l3out
    name: interface_group_policy_1
    state: absent
"""

RETURN = r"""
"""

import copy
from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.mso.plugins.module_utils.mso import (
    MSOModule,
    mso_argument_spec,
)
from ansible_collections.cisco.mso.plugins.module_utils.template import (
    MSOTemplate,
    KVPair,
)
from ansible_collections.cisco.mso.plugins.module_utils.utils import (
    append_update_ops_data,
    check_if_all_elements_are_none,
)
from ansible_collections.cisco.mso.plugins.module_utils.constants import QOS_LEVEL


def main():
    argument_spec = mso_argument_spec()
    argument_spec.update(
        template=dict(type="str", aliases=["l3out_template"]),
        template_id=dict(type="str", aliases=["l3out_template_id"]),
        l3out=dict(type="str", aliases=["l3out_name"]),
        l3out_uuid=dict(type="str"),
        description=dict(type="str"),
        name=dict(type="str", aliases=["l3out_interface_group_policy"]),
        interface_routing_policy=dict(
            type="dict",
            options=dict(
                name=dict(type="str"),
                template=dict(type="str"),
                template_id=dict(type="str"),
            ),
            required_by={
                "template": "name",
                "template_id": "name",
            },
            mutually_exclusive=[
                ["template", "template_id"],
            ],
        ),
        interface_routing_policy_uuid=dict(type="str"),
        bfd=dict(
            type="dict",
            options=dict(
                enable=dict(type="bool"),
                authentication=dict(type="bool"),
                key_id=dict(type="int"),
                key=dict(type="str", no_log=True),
            ),
        ),
        bfd_multi_hop=dict(
            type="dict",
            options=dict(
                enable=dict(type="bool"),
                authentication=dict(type="bool"),
                key_id=dict(type="int"),
                key=dict(type="str", no_log=True),
            ),
        ),
        ospf=dict(
            type="dict",
            options=dict(
                enable=dict(type="bool"),
                authentication_type=dict(type="str", choices=["simple", "md5", "none"]),
                key_id=dict(type="int"),
                key=dict(type="str", no_log=True),
            ),
        ),
        qos_priority=dict(type="str", choices=QOS_LEVEL),
        custom_qos_policy=dict(
            type="dict",
            options=dict(
                name=dict(type="str"),
                template=dict(type="str"),
                template_id=dict(type="str"),
            ),
            required_by={
                "template": "name",
                "template_id": "name",
            },
            mutually_exclusive=[
                ["template", "template_id"],
            ],
        ),
        custom_qos_policy_uuid=dict(type="str"),
        state=dict(type="str", default="query", choices=["absent", "query", "present"]),
    )

    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
        required_if=[
            ["state", "absent", ["name"]],
            ["state", "present", ["name"]],
        ],
        required_one_of=[
            ["template", "template_id"],
            ["l3out", "l3out_uuid"],
        ],
        mutually_exclusive=[
            ["template", "template_id"],
            ["l3out", "l3out_uuid"],
            ["interface_routing_policy", "interface_routing_policy_uuid"],
            ["custom_qos_policy", "custom_qos_policy_uuid"],
        ],
    )

    mso = MSOModule(module)

    template = module.params.get("template")
    template_id = module.params.get("template_id")
    l3out = module.params.get("l3out")
    l3out_uuid = module.params.get("l3out_uuid")
    name = module.params.get("name")
    description = module.params.get("description")
    interface_routing_policy = module.params.get("interface_routing_policy")
    interface_routing_policy_uuid = module.params.get("interface_routing_policy_uuid")
    bfd = module.params.get("bfd")
    bfd_multi_hop = module.params.get("bfd_multi_hop")
    ospf = module.params.get("ospf")
    qos_priority = module.params.get("qos_priority")
    custom_qos_policy = module.params.get("custom_qos_policy")
    custom_qos_policy_uuid = module.params.get("custom_qos_policy_uuid")
    state = module.params.get("state")

    mso_template = MSOTemplate(mso, "l3out", template, template_id)
    mso_template.validate_template("l3out")
    object_description = "L3Out Interface Group Policy"

    l3out_object = mso_template.get_l3out_object(l3out_uuid, l3out, True)
    existing_interface_groups = l3out_object.details.get("interfaceGroups", [])
    mso.check_template_when_name_is_provided(interface_routing_policy)
    mso.check_template_when_name_is_provided(custom_qos_policy)
    reference_dict = {
        "qos": {
            "name": "qosName",
            "reference": "qosRef",
            "type": "qos",
            "template": "qosTemplateName",
            "templateId": "qosTemplateId",
        },
        "interfaceRoutingPolicy": {
            "name": "interfaceRoutingPolicyName",
            "reference": "interfaceRoutingPolicyRef",
            "type": "l3OutIntfPolGroup",
            "template": "interfaceRoutingPolicyTemplateName",
            "templateId": "interfaceRoutingPolicyTemplateId",
        },
    }
    if name:
        match = mso_template.get_object_by_key_value_pairs(
            object_description,
            existing_interface_groups,
            [KVPair("name", name)],
        )
        if match:
            mso_template.update_config_with_template_and_references(match.details, reference_dict)
            mso.existing = mso.previous = copy.deepcopy(match.details)
    else:
        mso.existing = mso.previous = [
            mso_template.update_config_with_template_and_references(interface_group, reference_dict) for interface_group in existing_interface_groups
        ]

    if state != "query":
        interface_group_policy_path = "/l3outTemplate/l3outs/{0}/interfaceGroups/{1}".format(l3out_object.index, match.index if match else "-")

    ops = []

    protocols_dict = {"bfd": bfd, "bfdMultiHop": bfd_multi_hop, "ospf": ospf}
    empty_interface_routing_policy = False
    empty_custom_qos_policy = False

    if state == "present":
        check_authentication_requirements(mso, protocols_dict)
        mso_values = dict(
            name=name,
            description=description,
            interfaceRoutingPolicyRef=interface_routing_policy_uuid,
            qosPriority=qos_priority,
            qosRef=custom_qos_policy_uuid,
        )

        use_same_instance = (
            interface_routing_policy
            and custom_qos_policy
            and (
                (interface_routing_policy.get("template") is not None and interface_routing_policy.get("template") == custom_qos_policy.get("template"))
                or (
                    interface_routing_policy.get("template_id") is not None
                    and interface_routing_policy.get("template_id") == custom_qos_policy.get("template_id")
                )
            )
        )

        if use_same_instance:
            tenant_template = MSOTemplate(mso, "tenant", interface_routing_policy.get("template"), interface_routing_policy.get("template_id"))
            tenant_template.validate_template("tenantPolicy")
            process_tenant_policy(
                mso_values,
                tenant_template,
                interface_routing_policy,
                "interfaceRoutingPolicyRef",
                "l3OutIntfPolGroups",
            )
            process_tenant_policy(mso_values, tenant_template, custom_qos_policy, "qosRef", "qosPolicies")
        else:
            if interface_routing_policy:
                empty_interface_routing_policy = check_if_all_elements_are_none(list(interface_routing_policy.values()))
                if not empty_interface_routing_policy:
                    tenant_template_interface = MSOTemplate(
                        mso, "tenant", interface_routing_policy.get("template"), interface_routing_policy.get("template_id")
                    )
                    tenant_template_interface.validate_template("tenantPolicy")
                    process_tenant_policy(mso_values, tenant_template_interface, interface_routing_policy, "interfaceRoutingPolicyRef", "l3OutIntfPolGroups")

            if custom_qos_policy:
                empty_custom_qos_policy = check_if_all_elements_are_none(list(custom_qos_policy.values()))
                if not empty_custom_qos_policy:
                    tenant_template_qos = MSOTemplate(mso, "tenant", custom_qos_policy.get("template"), custom_qos_policy.get("template_id"))
                    tenant_template_qos.validate_template("tenantPolicy")
                    process_tenant_policy(mso_values, tenant_template_qos, custom_qos_policy, "qosRef", "qosPolicies")

        if match:
            mso_values_remove = list()

            if (interface_routing_policy_uuid == "" or empty_interface_routing_policy) and match.details.get("interfaceRoutingPolicyRef"):
                mso_values_remove.append("interfaceRoutingPolicyRef")
            if (custom_qos_policy_uuid == "" or empty_custom_qos_policy) and match.details.get("qosRef"):
                mso_values_remove.append("qosRef")

            for protocol_name, protocol_dict in protocols_dict.items():
                if protocol_dict is not None:
                    attribute_is_empty = check_if_all_elements_are_none(list(protocol_dict.values()))

                    if match.details.get(protocol_name, {}).get("key", {}).get("ref"):
                        match.details[protocol_name]["key"].pop("ref", None)
                        mso.previous[protocol_name]["key"].pop("ref", None)
                        mso.existing[protocol_name]["key"].pop("ref", None)

                    if attribute_is_empty and match.details.get(protocol_name):
                        mso_values_remove.append(protocol_name)

                    elif not attribute_is_empty and not match.details.get(protocol_name):
                        update_mso_values_attributes_payload(mso_values, {protocol_name: protocol_dict})

                    elif not attribute_is_empty and match.details.get(protocol_name):
                        mso_values[(protocol_name, "enabled")] = protocol_dict.get("enable")
                        if protocol_dict.get("enable") is True:
                            if protocol_name == "ospf":
                                mso_values[(protocol_name, "authType")] = protocol_dict.get("authentication_type")
                            else:
                                mso_values[(protocol_name, "authEnabled")] = protocol_dict.get("authentication")
                            mso_values[(protocol_name, "keyID")] = protocol_dict.get("key_id")

                            if protocol_dict.get("key") is not None:
                                if match.details.get(protocol_name, {}).get("key") is not None:
                                    mso_values[(protocol_name, "key", "value")] = protocol_dict.get("key")
                                else:
                                    mso_values[(protocol_name, "key")] = dict(value=protocol_dict.get("key"))
                        else:
                            if protocol_name == "ospf" and match.details.get(protocol_name, {}).get("authType"):
                                mso_values_remove.extend([(protocol_name, "key"), (protocol_name, "keyID"), (protocol_name, "authType")])
                            elif match.details.get(protocol_name, {}).get("authEnabled"):
                                mso_values_remove.extend([(protocol_name, "key"), (protocol_name, "keyID"), (protocol_name, "authEnabled")])

            append_update_ops_data(ops, match.details, interface_group_policy_path, mso_values, mso_values_remove)
            mso.sanitize(match.details, collate=True)
        else:
            update_mso_values_attributes_payload(mso_values, protocols_dict)
            mso.sanitize(mso_values)
            ops.append(dict(op="add", path=interface_group_policy_path, value=mso_values))

    elif state == "absent":
        if mso.existing:
            ops.append(dict(op="remove", path=interface_group_policy_path))

    if not module.check_mode and ops:
        mso_template.template = mso.request(mso_template.template_path, method="PATCH", data=ops)
        l3out_object = mso_template.get_l3out_object(l3out_uuid, l3out, True)
        interface_groups = l3out_object.details.get("interfaceGroups", [])
        match = mso_template.get_object_by_key_value_pairs(
            object_description,
            interface_groups,
            [KVPair("name", name)],
        )
        if match:
            for protocol_name in protocols_dict.keys():
                if match.details.get(protocol_name, {}).get("key", {}).get("ref"):
                    match.details[protocol_name]["key"].pop("ref", None)
            mso_template.update_config_with_template_and_references(match.details, reference_dict)
            mso.existing = match.details
        else:
            mso.existing = {}  # When the state is absent
    elif module.check_mode and state != "query":  # When the state is present/absent with check mode
        mso_template.update_config_with_template_and_references(mso.proposed, reference_dict)
        mso.existing = mso.proposed if state == "present" else {}

    mso.exit_json()


def check_authentication_requirements(mso, protocols_dict):
    failed_attributes = []

    for protocol_name, protocol_dict in protocols_dict.items():
        if protocol_dict is not None:
            if (
                protocol_dict.get("enable") is True
                and (protocol_dict.get("authentication") is True or protocol_dict.get("authentication_type") in ["md5", "simple"])
                and (protocol_dict.get("key") is None or protocol_dict.get("key_id") is None)
            ):
                failed_attributes.append("key and key_id are required under {0}".format(protocol_name))
            elif (protocol_dict.get("enable") is not True) and (
                protocol_dict.get("authentication") is not None or protocol_dict.get("key") is not None or protocol_dict.get("key_id") is not None
            ):
                failed_attributes.append("{0} must be enabled in order to use authentication, key and key_id".format(protocol_name))

    if failed_attributes:
        failure_messages = " | ".join(failed_attributes)
        mso.fail_json(msg=failure_messages)


def update_mso_values_attributes_payload(mso_values, protocols_dict):
    for protocol_name, protocol_dict in protocols_dict.items():
        if protocol_dict is not None:
            if protocol_dict.get("enable") is not None:
                mso_values[protocol_name] = {
                    "enabled": protocol_dict.get("enable"),
                }
            if protocol_dict.get("enable") is True:
                if protocol_dict.get("authentication") is True:
                    mso_values[protocol_name].update(
                        {
                            "authEnabled": protocol_dict.get("authentication"),
                            "keyID": protocol_dict.get("key_id"),
                            "key": {"value": protocol_dict.get("key")},
                        }
                    )
                elif protocol_dict.get("authentication_type") in ["md5", "simple"]:
                    mso_values[protocol_name].update(
                        {
                            "authType": protocol_dict.get("authentication_type"),
                            "keyID": protocol_dict.get("key_id"),
                            "key": {"value": protocol_dict.get("key")},
                        }
                    )


def process_tenant_policy(mso_values, tenant_template, policy, policy_ref, policy_type):
    mso_values[policy_ref] = tenant_template.get_template_policy_uuid("tenant", policy.get("name"), policy_type)


if __name__ == "__main__":
    main()



# Copyright (C) 2016 maxn nikolaev.makc@gmail.com
# Copyright (c) 2017 Ansible Project
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
author: Unknown (!UNKNOWN)
name: jabber
type: notification
short_description: Post task events to a Jabber server
description:
  - The chatty part of ChatOps with a Hipchat server as a target.
  - This callback plugin sends status updates to a HipChat channel during playbook execution.
requirements:
  - xmpp (Python library U(https://github.com/ArchipelProject/xmpppy))
options:
  server:
    description: Connection info to Jabber server.
    type: str
    required: true
    env:
      - name: JABBER_SERV
  user:
    description: Jabber user to authenticate as.
    type: str
    required: true
    env:
      - name: JABBER_USER
  password:
    description: Password for the user to the Jabber server.
    type: str
    required: true
    env:
      - name: JABBER_PASS
  to:
    description: Chat identifier that receives the message.
    type: str
    required: true
    env:
      - name: JABBER_TO
"""

import os

HAS_XMPP = True
try:
    import xmpp
except ImportError:
    HAS_XMPP = False

from ansible.plugins.callback import CallbackBase


class CallbackModule(CallbackBase):
    CALLBACK_VERSION = 2.0
    CALLBACK_TYPE = "notification"
    CALLBACK_NAME = "community.general.jabber"
    CALLBACK_NEEDS_WHITELIST = True

    def __init__(self, display=None):
        super().__init__(display=display)

        if not HAS_XMPP:
            self._display.warning(
                "The required python xmpp library (xmpppy) is not installed. "
                "pip install git+https://github.com/ArchipelProject/xmpppy"
            )
            self.disabled = True

        self.serv = os.getenv("JABBER_SERV")
        self.j_user = os.getenv("JABBER_USER")
        self.j_pass = os.getenv("JABBER_PASS")
        self.j_to = os.getenv("JABBER_TO")

        if (self.j_user or self.j_pass or self.serv or self.j_to) is None:
            self.disabled = True
            self._display.warning(
                "Jabber CallBack wants the JABBER_SERV, JABBER_USER, JABBER_PASS and JABBER_TO environment variables"
            )

    def send_msg(self, msg):
        """Send message"""
        jid = xmpp.JID(self.j_user)
        client = xmpp.Client(self.serv, debug=[])
        client.connect(server=(self.serv, 5222))
        client.auth(jid.getNode(), self.j_pass, resource=jid.getResource())
        message = xmpp.Message(self.j_to, msg)
        message.setAttr("type", "chat")
        client.send(message)
        client.disconnect()

    def v2_runner_on_ok(self, result):
        self._clean_results(result._result, result._task.action)
        self.debug = self._dump_results(result._result)

    def v2_playbook_on_task_start(self, task, is_conditional):
        self.task = task

    def v2_playbook_on_play_start(self, play):
        """Display Playbook and play start messages"""
        self.play = play
        name = play.name
        self.send_msg(f"Ansible starting play: {name}")

    def playbook_on_stats(self, stats):
        name = self.play
        hosts = sorted(stats.processed.keys())
        failures = False
        unreachable = False
        for h in hosts:
            s = stats.summarize(h)
            if s["failures"] > 0:
                failures = True
            if s["unreachable"] > 0:
                unreachable = True

        if failures or unreachable:
            out = self.debug
            self.send_msg(f"{name}: Failures detected \n{self.task} \nHost: {h}\n Failed at:\n{out}")
        else:
            out = self.debug
            self.send_msg(f"Great! \n Playbook {name} completed:\n{s} \n Last task debug:\n {out}")



# -*- coding: utf-8 -*-
# Copyright 2024 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function


__metaclass__ = type

"""
The Vrf_address_family parser templates file. This contains
a list of parser definitions and associated functions that
facilitates both facts gathering and native command generation for
the given network resource.
"""

import re

from ansible_collections.ansible.netcommon.plugins.module_utils.network.common.rm_base.network_template import (
    NetworkTemplate,
)


def _tmplt_maximum(maximum):
    cmd = "maximum routes"
    maxData = maximum.get("maximum")
    if maxData.get("max_routes"):
        cmd += f" {maxData['max_routes']}"
    if maxData.get("max_route_options", {}).get("threshold", {}).get("threshold_value"):
        threshold = maxData["max_route_options"]["threshold"]
        cmd += f" {threshold['threshold_value']}"
        if threshold.get("reinstall_threshold"):
            cmd += f" reinstall {threshold['reinstall_threshold']}"
    if maxData.get("max_route_options", {}).get("warning_only"):
        cmd += " warning-only"
    return cmd


def _tmplt_export_vrf(vrf):
    cmd = "export vrf"
    vrfData = vrf.get("vrf")
    if vrfData.get("max_prefix") or vrfData.get("map_import"):
        cmd += " default"
        if vrfData.get("max_prefix"):
            cmd += f" {vrfData['max_prefix']}"
        if vrfData.get("map_import"):
            cmd += f" map {vrfData['map_import']}"
    if vrfData.get("allow_vpn"):
        cmd += " allow-vpn"
    return cmd


def _tmplt_import_vrf(vrf):
    cmd = "import vrf"
    vrfData = vrf.get("vrf")
    if vrfData.get("max_prefix") or vrfData.get("map_import"):
        cmd += " default"
        if vrfData.get("max_prefix"):
            cmd += f" {vrfData['max_prefix']}"
        if vrfData.get("map_import"):
            cmd += f" map {vrfData['map_import']}"
    if vrfData.get("advertise_vpn"):
        cmd += " advertise-vpn"
    return cmd


class Vrf_address_familyTemplate(NetworkTemplate):
    def __init__(self, lines=None, module=None):
        super(Vrf_address_familyTemplate, self).__init__(lines=lines, tmplt=self, module=module)

    # fmt: off
    PARSERS = [
        {
            "name": "address_family",
            "getval": re.compile(
                r"""
                ^vrf\scontext\s(?P<name>\S+)\s+
                (?P<address_families>\s+address-family
                \s(?P<afi>\S+)\s(?P<safi>\S+))
                $""", re.VERBOSE,
            ),
            "setval": "address-family {{ afi }} {{ safi }}",
            "result": {
                '{{ name }}': {
                    "name": "{{ name }}",
                    "address_families": {
                        "{{ 'address_families_'+afi+'_'+safi }}": {
                            "afi": "{{ afi }}",
                            "safi": "{{ safi }}",
                        },
                    },
                },
            },
            "shared": True,
        },
        {
            "name": "maximum",
            "getval": re.compile(
                r"""
                \s+maximum\sroutes\s(?P<max_routes>\d+)
                (\s(?P<threshold_value>\d+))?
                (\sreinstall\s(?P<reinstall>\d+))?
                (\s((?P<warning_only>warning-only)))?
                $""", re.VERBOSE,
            ),
            "setval": _tmplt_maximum,
            "result": {
                '{{ name }}': {
                    "address_families": {
                        "{{ 'address_families_'+afi+'_'+safi }}": {
                            "afi": "{{ afi }}",
                            "safi": "{{ safi }}",
                            "maximum": {
                                "max_routes": "{{ max_routes }}",
                                "max_route_options": {
                                    "warning_only": "{{ True if warning_only }}",
                                    "threshold": {
                                        "threshold_value": "{{ threshold_value }}",
                                        "reinstall_threshold": "{{ reinstall }}",
                                    },
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "route_target.import",
            "getval": re.compile(
                r"""
                \s+route-target\simport\s(?P<import>\S+)
                $""", re.VERBOSE,
            ),
            "setval": "route-target import {{ import }}",
            "result": {
                '{{ name }}': {
                    "address_families": {
                        "{{ 'address_families_'+afi+'_'+safi }}": {
                            "afi": "{{ afi }}",
                            "safi": "{{ safi }}",
                            "route_target": [{
                                "import": "{{ import }}",
                            }],
                        },
                    },
                },
            },
        },
        {
            "name": "route_target.export",
            "getval": re.compile(
                r"""
                \s+route-target\sexport\s(?P<export>\S+)
                $""", re.VERBOSE,
            ),
            "setval": "route-target export {{ export }}",
            "result": {
                '{{ name }}': {
                    "address_families": {
                        "{{ 'address_families_'+afi+'_'+safi }}": {
                            "afi": "{{ afi }}",
                            "safi": "{{ safi }}",
                            "route_target": [{
                                "export": "{{ export }}",
                            }],
                        },
                    },
                },
            },
        },
        {
            "name": "export.map",
            "getval": re.compile(
                r"""
                \s+export\smap\s(?P<export_map>\S+)
                $""", re.VERBOSE,
            ),
            "setval": "export map {{ map }}",
            "result": {
                '{{ name }}': {
                    "address_families": {
                        "{{ 'address_families_'+afi+'_'+safi }}": {
                            "afi": "{{ afi }}",
                            "safi": "{{ safi }}",
                            "export": [{
                                "map": "{{ export_map }}",
                            }],
                        },
                    },
                },
            },
        },
        {
            "name": "export.vrf",
            "getval": re.compile(
                r"""
                \s+export\svrf
                ((\sdefault)
                (\s(?P<max_prefix>\d+))?
                (\smap\s(?P<map_import>\S+))?)?
                (\s(?P<allow_vpn>allow-vpn))?
                $""", re.VERBOSE,
            ),
            "setval": _tmplt_export_vrf,
            "result": {
                '{{ name }}': {
                    "address_families": {
                        "{{ 'address_families_'+afi+'_'+safi }}": {
                            "afi": "{{ afi }}",
                            "safi": "{{ safi }}",
                            "export": [{
                                "vrf": {
                                    "allow_vpn": "{{ True if allow_vpn }}",
                                    "max_prefix": "{{ max_prefix if max_prefix }}",
                                    "map_import": "{{ map_import if map_import }}",
                                },
                            }],
                        },
                    },
                },
            },
        },
        {
            "name": "import.map",
            "getval": re.compile(
                r"""
                \s+import\smap\s(?P<import_map>\S+)
                $""", re.VERBOSE,
            ),
            "setval": "import map {{ map }}",
            "result": {
                '{{ name }}': {
                    "address_families": {
                        "{{ 'address_families_'+afi+'_'+safi }}": {
                            "afi": "{{ afi }}",
                            "safi": "{{ safi }}",
                            "import": [{
                                "map": "{{ import_map }}",
                            }],
                        },
                    },
                },
            },
        },
        {
            "name": "import.vrf",
            "getval": re.compile(
                r"""
                \s+import\svrf
                ((\sdefault)
                (\s(?P<max_prefix>\d+))?
                (\smap\s(?P<map_import>\S+))?)?
                (\s(?P<advertise_vpn>advertise-vpn))?
                $""", re.VERBOSE,
            ),
            "setval": _tmplt_import_vrf,
            "result": {
                '{{ name }}': {
                    "address_families": {
                        "{{ 'address_families_'+afi+'_'+safi }}": {
                            "afi": "{{ afi }}",
                            "safi": "{{ safi }}",
                            "import": [{
                                "vrf": {
                                    "advertise_vpn": "{{ True if advertise_vpn }}",
                                    "max_prefix": "{{ max_prefix if max_prefix }}",
                                    "map_import": "{{ map_import if map_import }}",
                                },
                            }],
                        },
                    },
                },
            },
        },
    ]
    # fmt: on



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2017 Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
module: waf_web_acl
short_description: Create and delete WAF Web ACLs
version_added: 1.0.0
description:
  - Module for WAF classic, for WAF v2 use the I(wafv2_*) modules.
  - Read the AWS documentation for WAF U(https://docs.aws.amazon.com/waf/latest/developerguide/classic-waf-chapter.html).
  - Prior to release 5.0.0 this module was called C(community.aws.aws_waf_web_acl).
    The usage did not change.

author:
  - Mike Mochan (@mmochan)
  - Will Thames (@willthames)

deprecated:
  removed_in: 12.0.0
  why: Amazon has retired the "WAF Classic" service.
  alternative: Please use the "AWS WAF (WAFv2)" service and modules instead.

options:
  name:
    description: Name of the Web Application Firewall ACL to manage.
    required: true
    type: str
  default_action:
    description: The action that you want AWS WAF to take when a request doesn't
      match the criteria specified in any of the Rule objects that are associated with the WebACL.
    choices:
      - block
      - allow
      - count
    type: str
  state:
    description: Whether the Web ACL should be present or absent.
    choices:
      - present
      - absent
    default: present
    type: str
  metric_name:
    description:
      - A friendly name or description for the metrics for this WebACL.
      - The name can contain only alphanumeric characters (A-Z, a-z, 0-9); the name can't contain whitespace.
      - You can't change I(metric_name) after you create the WebACL.
      - Metric name will default to I(name) with disallowed characters stripped out.
    type: str
  rules:
    description:
      - A list of rules that the Web ACL will enforce.
    type: list
    elements: dict
    suboptions:
      name:
        description: Name of the rule.
        type: str
        required: true
      action:
        description: The action to perform.
        type: str
        required: true
      priority:
        description: The priority of the action.  Priorities must be unique. Lower numbered priorities are evaluated first.
        type: int
        required: true
      type:
        description: The type of rule.
        choices:
          - rate_based
          - regular
        type: str
  purge_rules:
    description:
      - Whether to remove rules that aren't passed with I(rules).
    default: False
    type: bool
  waf_regional:
    description: Whether to use C(waf-regional) module.
    default: false
    required: false
    type: bool

extends_documentation_fragment:
  - amazon.aws.common.modules
  - amazon.aws.region.modules
  - amazon.aws.boto3
"""

EXAMPLES = r"""
- name: create web ACL
  community.aws.waf_web_acl:
    name: my_web_acl
    rules:
      - name: my_rule
        priority: 1
        action: block
    default_action: block
    purge_rules: true
    state: present

- name: delete the web acl
  community.aws.waf_web_acl:
    name: my_web_acl
    state: absent
"""

RETURN = r"""
web_acl:
  description: contents of the Web ACL.
  returned: always
  type: complex
  contains:
    default_action:
      description: Default action taken by the Web ACL if no rules match.
      returned: always
      type: dict
      sample:
        type: BLOCK
    metric_name:
      description: Metric name used as an identifier.
      returned: always
      type: str
      sample: mywebacl
    name:
      description: Friendly name of the Web ACL.
      returned: always
      type: str
      sample: my web acl
    rules:
      description: List of rules.
      returned: always
      type: complex
      contains:
        action:
          description: Action taken by the WAF when the rule matches.
          returned: always
          type: complex
          sample:
            type: ALLOW
        priority:
          description: priority number of the rule (lower numbers are run first).
          returned: always
          type: int
          sample: 2
        rule_id:
          description: Rule ID.
          returned: always
          type: str
          sample: a6fc7ab5-287b-479f-8004-7fd0399daf75
        type:
          description: Type of rule (either REGULAR or RATE_BASED).
          returned: always
          type: str
          sample: REGULAR
    web_acl_id:
      description: Unique identifier of Web ACL.
      returned: always
      type: str
      sample: 10fff965-4b6b-46e2-9d78-24f6d2e2d21c
"""

import re

try:
    import botocore
except ImportError:
    pass  # handled by AnsibleAWSModule

from ansible.module_utils.common.dict_transformations import camel_dict_to_snake_dict

from ansible_collections.amazon.aws.plugins.module_utils.waf import list_regional_rules_with_backoff
from ansible_collections.amazon.aws.plugins.module_utils.waf import list_regional_web_acls_with_backoff
from ansible_collections.amazon.aws.plugins.module_utils.waf import list_rules_with_backoff
from ansible_collections.amazon.aws.plugins.module_utils.waf import list_web_acls_with_backoff
from ansible_collections.amazon.aws.plugins.module_utils.waf import run_func_with_change_token_backoff
from ansible_collections.amazon.aws.plugins.module_utils.waiters import get_waiter

from ansible_collections.community.aws.plugins.module_utils.modules import AnsibleCommunityAWSModule as AnsibleAWSModule


def get_web_acl_by_name(client, module, name):
    acls = [d["WebACLId"] for d in list_web_acls(client, module) if d["Name"] == name]
    if acls:
        return acls[0]
    else:
        return acls


def create_rule_lookup(client, module):
    if client.__class__.__name__ == "WAF":
        try:
            rules = list_rules_with_backoff(client)
            return dict((rule["Name"], rule) for rule in rules)
        except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
            module.fail_json_aws(e, msg="Could not list rules")
    elif client.__class__.__name__ == "WAFRegional":
        try:
            rules = list_regional_rules_with_backoff(client)
            return dict((rule["Name"], rule) for rule in rules)
        except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
            module.fail_json_aws(e, msg="Could not list regional rules")


def get_web_acl(client, module, web_acl_id):
    try:
        return client.get_web_acl(WebACLId=web_acl_id)["WebACL"]
    except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
        module.fail_json_aws(e, msg=f"Could not get Web ACL with id {web_acl_id}")


def list_web_acls(
    client,
    module,
):
    if client.__class__.__name__ == "WAF":
        try:
            return list_web_acls_with_backoff(client)
        except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
            module.fail_json_aws(e, msg="Could not get Web ACLs")
    elif client.__class__.__name__ == "WAFRegional":
        try:
            return list_regional_web_acls_with_backoff(client)
        except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
            module.fail_json_aws(e, msg="Could not get Web ACLs")


def find_and_update_web_acl(client, module, web_acl_id):
    acl = get_web_acl(client, module, web_acl_id)
    rule_lookup = create_rule_lookup(client, module)
    existing_rules = acl["Rules"]
    desired_rules = [
        {
            "RuleId": rule_lookup[rule["name"]]["RuleId"],
            "Priority": rule["priority"],
            "Action": {"Type": rule["action"].upper()},
            "Type": rule.get("type", "regular").upper(),
        }
        for rule in module.params["rules"]
    ]
    missing = [rule for rule in desired_rules if rule not in existing_rules]
    extras = []
    if module.params["purge_rules"]:
        extras = [rule for rule in existing_rules if rule not in desired_rules]

    insertions = [format_for_update(rule, "INSERT") for rule in missing]
    deletions = [format_for_update(rule, "DELETE") for rule in extras]
    changed = bool(insertions + deletions)

    # Purge rules before adding new ones in case a deletion shares the same
    # priority as an insertion.
    params = {"WebACLId": acl["WebACLId"], "DefaultAction": acl["DefaultAction"]}
    change_tokens = []
    if deletions:
        try:
            params["Updates"] = deletions
            result = run_func_with_change_token_backoff(client, module, params, client.update_web_acl)
            change_tokens.append(result["ChangeToken"])
            get_waiter(
                client,
                "change_token_in_sync",
            ).wait(ChangeToken=result["ChangeToken"])
        except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
            module.fail_json_aws(e, msg="Could not update Web ACL")
    if insertions:
        try:
            params["Updates"] = insertions
            result = run_func_with_change_token_backoff(client, module, params, client.update_web_acl)
            change_tokens.append(result["ChangeToken"])
        except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
            module.fail_json_aws(e, msg="Could not update Web ACL")
    if change_tokens:
        for token in change_tokens:
            get_waiter(
                client,
                "change_token_in_sync",
            ).wait(ChangeToken=token)
    if changed:
        acl = get_web_acl(client, module, web_acl_id)
    return changed, acl


def format_for_update(rule, action):
    return dict(
        Action=action,
        ActivatedRule=dict(
            Priority=rule["Priority"],
            RuleId=rule["RuleId"],
            Action=dict(Type=rule["Action"]["Type"]),
        ),
    )


def remove_rules_from_web_acl(client, module, web_acl_id):
    acl = get_web_acl(client, module, web_acl_id)
    deletions = [format_for_update(rule, "DELETE") for rule in acl["Rules"]]
    try:
        params = {"WebACLId": acl["WebACLId"], "DefaultAction": acl["DefaultAction"], "Updates": deletions}
        run_func_with_change_token_backoff(client, module, params, client.update_web_acl)
    except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
        module.fail_json_aws(e, msg="Could not remove rule")


def ensure_web_acl_present(client, module):
    changed = False
    result = None
    name = module.params["name"]
    web_acl_id = get_web_acl_by_name(client, module, name)
    if web_acl_id:
        (changed, result) = find_and_update_web_acl(client, module, web_acl_id)
    else:
        metric_name = module.params["metric_name"]
        if not metric_name:
            metric_name = re.sub(r"[^A-Za-z0-9]", "", module.params["name"])
        default_action = module.params["default_action"].upper()
        try:
            params = {"Name": name, "MetricName": metric_name, "DefaultAction": {"Type": default_action}}
            new_web_acl = run_func_with_change_token_backoff(client, module, params, client.create_web_acl)
        except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
            module.fail_json_aws(e, msg="Could not create Web ACL")
        (changed, result) = find_and_update_web_acl(client, module, new_web_acl["WebACL"]["WebACLId"])
    return changed, result


def ensure_web_acl_absent(client, module):
    web_acl_id = get_web_acl_by_name(client, module, module.params["name"])
    if web_acl_id:
        web_acl = get_web_acl(client, module, web_acl_id)
        if web_acl["Rules"]:
            remove_rules_from_web_acl(client, module, web_acl_id)
        try:
            run_func_with_change_token_backoff(
                client, module, {"WebACLId": web_acl_id}, client.delete_web_acl, wait=True
            )
            return True, {}
        except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
            module.fail_json_aws(e, msg="Could not delete Web ACL")
    return False, {}


def main():
    argument_spec = dict(
        name=dict(required=True),
        default_action=dict(choices=["block", "allow", "count"]),
        metric_name=dict(),
        state=dict(default="present", choices=["present", "absent"]),
        rules=dict(type="list", elements="dict"),
        purge_rules=dict(type="bool", default=False),
        waf_regional=dict(type="bool", default=False),
    )
    module = AnsibleAWSModule(
        argument_spec=argument_spec,
        required_if=[["state", "present", ["default_action", "rules"]]],
    )
    state = module.params.get("state")

    resource = "waf" if not module.params["waf_regional"] else "waf-regional"
    client = module.client(resource)
    if state == "present":
        (changed, results) = ensure_web_acl_present(client, module)
    else:
        (changed, results) = ensure_web_acl_absent(client, module)

    module.exit_json(changed=changed, web_acl=camel_dict_to_snake_dict(results))


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or
# https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
module: networks_wireless_zigbee
short_description: Resource module for networks _wireless _zigbee
description:
  - Manage operation update of the resource networks _wireless _zigbee.
  - Update Zigbee Configs for specified network.
version_added: '1.0.0'
extends_documentation_fragment:
  - cisco.meraki.module
author: Francisco Munoz (@fmunoz)
options:
  defaults:
    description: Default Settings for Zigbee Devices.
    suboptions:
      channel:
        description: Channel.
        type: str
      transmitPowerLevel:
        description: Transmit Power Level.
        type: int
    type: dict
  enabled:
    description: To enable/disable Zigbee on the network.
    type: bool
  iotController:
    description: Zigbee's IoT controller details.
    suboptions:
      serial:
        description: Device Serial number.
        type: str
    type: dict
  lockManagement:
    description: Login Credentials of on-premises lock management.
    suboptions:
      address:
        description: Valid hostname or IP address.
        type: str
      password:
        description: Password.
        type: str
      username:
        description: Username.
        type: str
    type: dict
  networkId:
    description: NetworkId path parameter. Network ID.
    type: str
requirements:
  - meraki >= 2.4.9
  - python >= 3.5
seealso:
  - name: Cisco Meraki documentation for wireless updateNetworkWirelessZigbee
    description: Complete reference of the updateNetworkWirelessZigbee API.
    link: https://developer.cisco.com/meraki/api-v1/#!update-network-wireless-zigbee
notes:
  - SDK Method used are
    wireless.Wireless.update_network_wireless_zigbee,
  - Paths used are
    put /networks/{networkId}/wireless/zigbee,
"""

EXAMPLES = r"""
- name: Update all
  cisco.meraki.networks_wireless_zigbee:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    defaults:
      channel: string
      transmitPowerLevel: 0
    enabled: true
    iotController:
      serial: string
    lockManagement:
      address: string
      password: string
      username: string
    networkId: string
"""
RETURN = r"""
meraki_response:
  description: A dictionary or list with the response returned by the Cisco Meraki Python SDK
  returned: always
  type: dict
  sample: >
    {
      "network": {
        "id": "string"
      },
      "enabled": true,
      "iotController": {
        "name": "string",
        "mac": "string",
        "serial": "string",
        "status": "string"
      },
      "lockManagement": {
        "address": "string",
        "username": "string",
        "status": "string"
      },
      "defaults": {
        "transmitPowerLevel": 0,
        "channel": "string"
      }
    }
"""



# (c) 2012-2014, Michael DeHaan <michael.dehaan@gmail.com>
# (c) 2015 Toshio Kuratomi <tkuratomi@ansible.com>
# (c) 2017, Peter Sprygada <psprygad@redhat.com>
# (c) 2017 Ansible Project
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import absolute_import, division, print_function


__metaclass__ = type

import os

from ansible import constants as C
from ansible.plugins.connection import ConnectionBase
from ansible.plugins.loader import connection_loader
from ansible.utils.display import Display
from ansible.utils.path import unfrackpath


display = Display()


__all__ = ["PersistentConnectionBase"]


class PersistentConnectionBase(ConnectionBase):
    """
    A base for simple persistent connections.
    """

    force_persistence = True
    # Do not use _remote_is_local in other connections
    _remote_is_local = True

    def __init__(self, play_context, new_stdin, *args, **kwargs):
        super(PersistentConnectionBase, self).__init__(play_context, new_stdin, *args, **kwargs)
        self._messages = []
        self._conn_closed = False

        self._local = connection_loader.get("local", play_context, "/dev/null")
        self._local.set_options()

        # reconstruct the socket_path and set instance values accordingly
        self._ansible_playbook_pid = kwargs.get("ansible_playbook_pid")
        self._update_connection_state()

    def exec_command(self, cmd, in_data=None, sudoable=True):
        return self._local.exec_command(cmd, in_data, sudoable)

    def queue_message(self, level, message):
        """
        Adds a message to the queue of messages waiting to be pushed back to the controller process.

        :arg level: A string which can either be the name of a method in display, or 'log'. When
            the messages are returned to task_executor, a value of log will correspond to
            ``display.display(message, log_only=True)``, while another value will call ``display.[level](message)``
        """
        self._messages.append((level, message))

    def pop_messages(self):
        messages, self._messages = self._messages, []
        return messages

    def put_file(self, in_path, out_path):
        """Transfer a file from local to remote"""
        return self._local.put_file(in_path, out_path)

    def fetch_file(self, in_path, out_path):
        """Fetch a file from remote to local"""
        return self._local.fetch_file(in_path, out_path)

    def reset(self):
        """
        Reset the connection
        """
        if self._socket_path:
            self.queue_message(
                "vvvv",
                "resetting persistent connection for socket_path %s" % self._socket_path,
            )
            self.close()
        self.queue_message("vvvv", "reset call on connection instance")

    def close(self):
        self._conn_closed = True
        if self._connected:
            self._connected = False

    def _update_connection_state(self):
        """
        Reconstruct the connection socket_path and check if it exists

        If the socket path exists then the connection is active and set
        both the _socket_path value to the path and the _connected value
        to True.  If the socket path doesn't exist, leave the socket path
        value to None and the _connected value to False
        """
        ssh = connection_loader.get("ssh", class_only=True)
        control_path = ssh._create_control_path(
            self._play_context.remote_addr,
            self._play_context.port,
            self._play_context.remote_user,
            self._play_context.connection,
            self._ansible_playbook_pid,
        )

        tmp_path = unfrackpath(C.PERSISTENT_CONTROL_PATH_DIR)
        socket_path = unfrackpath(control_path % dict(directory=tmp_path))

        if os.path.exists(socket_path):
            self._connected = True
            self._socket_path = socket_path

    def _log_messages(self, message):
        if self.get_option("persistent_log_messages"):
            self.queue_message("log", message)



#!/usr/bin/python
# -*- coding: utf-8 -*-

# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['preview'],
                    'supported_by': 'community'}

DOCUMENTATION = r'''
---
module: intersight_macsec_policy_info
short_description: Gather information about MACsec Policies in Cisco Intersight
description:
  - Gather information about MACsec Policies in L(Cisco Intersight,https://intersight.com).
  - Information can be filtered by O(organization) and O(name).
  - If no filters are passed, all MACsec Policies will be returned.
extends_documentation_fragment: intersight
options:
  organization:
    description:
      - The name of the organization the MACsec Policy belongs to.
    type: str
  name:
    description:
      - The name of the MACsec Policy to gather information from.
    type: str
author:
  - Ron Gershburg (@rgershbu)
'''

EXAMPLES = r'''
- name: Fetch a specific MACsec Policy by name
  cisco.intersight.intersight_macsec_policy_info:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    name: "macsec-policy-01"

- name: Fetch all MACsec Policies in a specific Organization
  cisco.intersight.intersight_macsec_policy_info:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    organization: "default"

- name: Fetch all MACsec Policies
  cisco.intersight.intersight_macsec_policy_info:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
'''

RETURN = r'''
api_response:
  description: The API response output returned by the specified resource.
  returned: always
  type: dict
  sample:
    "api_response": [
      {
        "Name": "macsec-policy-01",
        "ObjectType": "fabric.MacsecPolicy",
        "Description": "MACsec policy with primary keychain",
        "CipherSuite": "GCM-AES-XPN-256",
        "ConfidentialityOffset": "CONF-OFFSET-0",
        "SecurityPolicy": "Should-secure",
        "KeyServerPriority": 16,
        "SakExpiryTime": 0,
        "ReplayWindowSize": 148809600,
        "IncludeIcvIndicator": false,
        "MacSecEaPol": {
            "EaPolMacAddress": "0180.C200.0003",
            "EaPolEthertype": "0x888e"
        },
        "PrimaryKeyChain": {
            "Name": "primary-keychain",
            "SecKeys": [
                {
                    "Id": "1234",
                    "CryptographicAlgorithm": "AES_256_CMAC",
                    "KeyType": "Type-6",
                    "SendLifetimeUnlimited": true,
                    "SendLifetimeInfinite": false,
                    "IsOctetStringSet": true
                }
            ]
        },
        "FallbackKeyChain": {
            "Name": "",
            "SecKeys": null
        },
        "Tags": [
            {
                "Key": "Environment",
                "Value": "Production"
            }
        ]
      }
    ]
'''


from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.intersight.plugins.module_utils.intersight import IntersightModule, intersight_argument_spec


def main():
    argument_spec = intersight_argument_spec.copy()
    argument_spec.update(
        organization=dict(type='str'),
        name=dict(type='str')
    )
    module = AnsibleModule(
        argument_spec,
        supports_check_mode=True,
    )
    intersight = IntersightModule(module)
    intersight.result['api_response'] = {}
    intersight.result['trace_id'] = ''

    resource_path = '/fabric/MacSecPolicies'
    query_params = intersight.set_query_params()

    intersight.get_resource(
        resource_path=resource_path,
        query_params=query_params,
        return_list=True
    )
    module.exit_json(**intersight.result)


if __name__ == '__main__':
    main()



#!/usr/bin/python
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

DOCUMENTATION = '''
---
module: azure_rm_ml_data
version_added: "3.16.0"
short_description: Create, Update, Archive, Restore an Azure Machine Learning Data Asset
description:
    - Create, Update, Archive, Restore an Azure Machine Learning Data Asset.
options:
    name:
        description:
            - Name of the Azure ML data asset.
        type: str
        required: true
    resource_group:
        description:
            - Name of resource group.
        required: false
        type: str
    ml_workspace:
        description:
            - Name of the Azure ML workspace.
        required: false
        type: str
    ml_registry:
        description:
            - Name of the Azure ML registry.
        required: false
        type: str
    description:
        description:
            - Description of the data target.
        type: str
        required: false
    resource_definition:
        description:
            - "Local path to the YAML file containing the Azure ML data specification.
               The YAML reference docs for data can be found at:
               https://aka.ms/ml-cli-v2-data-yaml-reference."
        type: str
        required: false
    version:
        description:
            - Data Asset version.
        type: str
    label:
        description:
            - Label of the data asset. Must be provided, if version is not
              provided. Mutually exclusive with version.
        type: str
    data_type:
        description:
            - The type of managed identity.
        type: str
        choices:
            - mltable
            - uri_file
            - uri_folder
        required: false
    path:
        description:
            - Path to the data asset, can be local or remote.
        type: str
        required: false
    datastore:
        description:
            - The datastore to upload the local artifact to.
        type: str
        required: false
    skip_validation:
        description:
            - Skip validation of MLTable metadata when type is MLTable.
        type: bool
        required: false
    location:
        description:
            - The location to be used for the new data.
              If not specified, defaults to the location of the workspace.
        type: str
        required: false
    state:
        description:
            - State of the Data. Use C(present) to create
              or update. C(archive) to archive and C(restore)
              to restore.
        default: present
        type: str
        choices:
            - present
            - archive
            - restore

extends_documentation_fragment:
    - azure.azcollection.azure
    - azure.azcollection.azure_tags

author:
    - Bill Peck (@p3ck)
'''

EXAMPLES = '''
- name: Create ML Data
  azure.azcollection.azure_rm_ml_data:
    location: eastus
    name: MyData
    description: My Data created by Ansible
    resource_group: myResourceGroup
    ml_workspace: myMLWorkspace
    tags:
      createdByToolkit: Ansible
'''

RETURN = '''
changed:
    description:
        - Whether the resource is changed.
    returned: always
    type: bool
    sample: false
ml_data_asset:
    description:
        - Data that was just created or updated.
    returned: always
    type: dict
    sample: {
      "creation_context":
        {
          "created_at": "2026-03-05T20:28:42.120932+00:00",
          "created_by": "ca413070-9a06-4e0a-8bb8-c167d29fa09e",
          "created_by_type": "Application",
          "last_modified_at": "2026-03-05T20:28:57.237566+00:00",
          "last_modified_by": "ca413070-9a06-4e0a-8bb8-c167d29fa09e",
          "last_modified_by_type": "Application",
        },
      "description": "Data asset created from local folder.",
      "id": "/subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourceGroups/xxxxx-ml-data/providers/Microsoft.MachineLearningServices/workspaces/workspace-xxxxxxx-xxx/data/data-xxxxxxx-xxx/versions/2",
      "name": "data-xxxxxxx-xxx",
      "path": "azureml://subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourcegroups/xxxxx-ml-data/workspaces/workspace-xxxxxxx-xxx/datastores/workspaceblobstore/paths/LocalUpload/xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx/data/",
      "properties": {},
      "tags": {},
      "type": "uri_folder",
      "version": "2",
    }
'''  # NOQA


try:
    import io
    from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common_ml import MLClientCommon
    from azure.core.exceptions import ResourceNotFoundError
    from azure.ai.ml.entities._load_functions import load_data
except ImportError:
    # This is handled in azure_rm_common
    pass


class AzureRMMLData(MLClientCommon):

    UPDATE_OVERRIDES = ['description',
                        'tags'
                        ]

    PARAM_OVERRIDES = UPDATE_OVERRIDES + ['name',
                                          'data_type',
                                          'version',
                                          'label',
                                          'datastore',
                                          'path',
                                          'skip_validation',
                                          'location',
                                          ]

    def __init__(self):

        self.module_arg_spec = dict(
            name=dict(
                type='str',
                required=True,
            ),
            description=dict(
                type='str',
                required=False,
            ),
            resource_definition=dict(
                type='str',
                required=False,
            ),
            location=dict(
                type='str',
                required=False,
            ),
            resource_group=dict(
                type='str',
                required=False,
            ),
            ml_workspace=dict(
                type='str',
                required=False,
            ),
            ml_registry=dict(
                type='str',
                required=False,
            ),
            data_type=dict(
                type='str',
                required=False,
                choices=['mltable', 'uri_file', 'uri_folder'],
            ),
            version=dict(
                type='str',
                required=False,
            ),
            label=dict(
                type='str',
                required=False,
            ),
            path=dict(
                type='str',
                required=False,
            ),
            skip_validation=dict(
                type='bool',
            ),
            datastore=dict(
                type='str',
                required=False,
            ),
            state=dict(
                type='str',
                default='present',
                choices=['present', 'archive', 'restore'],
            ),
        )

        self._client = None
        self.ml_data_asset = None
        self.ml_registry = None
        self.state = None

        self.results = dict(
            changed=False,
            compare=[],
            ml_data_asset={}
        )

        required_one_of = [('ml_workspace', 'ml_registry')]
        mutually_exclusive = [('version', 'label'),
                              ('ml_workspace', 'ml_registry')]
        required_together = [('ml_workspace', 'resource_group')]

        super(AzureRMMLData, self).__init__(self.module_arg_spec,
                                            supports_check_mode=True,
                                            required_one_of=required_one_of,
                                            mutually_exclusive=mutually_exclusive,
                                            required_together=required_together)

    def exec_module(self, **kwargs):

        for key in list(self.module_arg_spec.keys()) + ['tags']:
            setattr(self, key, kwargs[key])

        if self.resource_definition:
            resource_definition = io.StringIO(self.resource_definition)
        else:
            resource_definition = None

        changed = False
        ml_data_asset_info = self.get(name=self.name,
                                      version=self.version,
                                      label=self.label)

        if self.state == 'present':
            if ml_data_asset_info:
                # Update
                params_override = self.update_params(kwargs,
                                                     self.UPDATE_OVERRIDES,
                                                     ml_data_asset_info)

                # Generate ml_data_asset based on ansible args passed in.
                ml_data_asset = load_data(None, params_override=params_override)
                ml_data_asset_info = self.entity_to_dict(ml_data_asset_info)
                changed = not self.default_compare({},
                                                   self.entity_to_dict(ml_data_asset),
                                                   ml_data_asset_info,
                                                   '',
                                                   self.results)
                if changed and not self.check_mode:
                    response = self.client.data.create_or_update(ml_data_asset)
                    ml_data_asset_info = self.entity_to_dict(response)

            else:
                # Create
                params_override = self.update_params(kwargs, self.PARAM_OVERRIDES)

                # Generate ml_data_asset based on ansible args passed in.
                ml_data_asset = load_data(resource_definition,
                                          params_override=params_override)
                changed = True
                if not self.check_mode:
                    response = self.client.data.create_or_update(ml_data_asset)
                    ml_data_asset_info = self.entity_to_dict(response)
        elif self.state == 'archive':
            # Archive
            list_view_type = self.get_list_view_type('active')
            results = self.client.data.list(list_view_type=list_view_type)
            ml_data_assets = [x.name for x in results]

            if self.name in ml_data_assets:
                changed = True
                if not self.check_mode:
                    self.archive(name=self.name,
                                 version=self.version,
                                 label=self.label)
                ml_data_asset_info = self.get(name=self.name,
                                              version=self.version,
                                              label=self.label,
                                              as_dict=True)
        elif self.state == 'restore':
            # Restore
            list_view_type = self.get_list_view_type('archived')
            results = self.client.data.list(list_view_type=list_view_type)
            ml_data_assets = [x.name for x in results]

            if self.name in ml_data_assets:
                changed = True
                if not self.check_mode:
                    self.restore(name=self.name,
                                 version=self.version,
                                 label=self.label)
                ml_data_asset_info = self.get(name=self.name,
                                              version=self.version,
                                              label=self.label,
                                              as_dict=True)

        self.results['ml_data_asset'] = ml_data_asset_info
        self.results['changed'] = changed
        return self.results

    def update_params(self, kwargs, param_overrides, ml_data_asset=None):
        params_override = []

        # If ml_data_asset is defined use those values as a base.
        # Update can't change some of these values but we need
        # to keep the same values as before the update.
        if ml_data_asset:
            items = self.entity_to_dict(ml_data_asset).items()
            params_override = [{k: v} for k, v in items if k != "ssh_settings"]

        # Update values based on what param_overrides says we can update.
        for key in list(self.module_arg_spec.keys()) + ['tags']:
            if kwargs[key] is not None and key in param_overrides:
                params_override.append({key: kwargs[key]})
        return params_override

    def get(self, name=None, version=None, label=None, as_dict=False):
        try:
            # Attempt to retrieve ml_data_asset based on name
            ml_data_asset_info = self.client.data.get(name=name,
                                                      version=version,
                                                      label=label)
            if as_dict:
                ml_data_asset_info = self.entity_to_dict(ml_data_asset_info)
        except ResourceNotFoundError:
            ml_data_asset_info = None
        except Exception as e:  # pylint: disable=broad-exception-caught
            ml_data_asset_info = None
        return ml_data_asset_info

    def archive(self, name=None, version=None, label=None):
        try:
            response = self.client.data.archive(name=name,
                                                version=version,
                                                label=label)
            return response
        except Exception as e:  # pylint: disable=broad-exception-caught
            self.fail("Error archive the data instance: {0}".format(str(e)))

    def restore(self, name=None, version=None, label=None):
        try:
            response = self.client.data.restore(name=name,
                                                version=version,
                                                label=label)
            return response
        except Exception as e:  # pylint: disable=broad-exception-caught
            self.fail("Error restore the data instance: {0}".format(str(e)))


def main():
    AzureRMMLData()


if __name__ == '__main__':
    main()



#!/usr/bin/python
#
# This file is part of Ansible
#
# Ansible is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Ansible is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.
#
from __future__ import absolute_import, division, print_function


__metaclass__ = type


DOCUMENTATION = """
module: eos_eapi
author: Peter Sprygada (@privateip)
short_description: Manage and configure Arista EOS eAPI.
requirements:
- EOS v4.12 or greater
description:
- Use to enable or disable eAPI access, and set the port and state of http, https,
  local_http and unix-socket servers.
- When enabling eAPI access the default is to enable HTTP on port 80, enable HTTPS
  on port 443, disable local HTTP, and disable Unix socket server. Use the options
  listed below to override the default configuration.
- Requires EOS v4.12 or greater.
version_added: 1.0.0
options:
  http:
    description:
    - The C(http) argument controls the operating state of the HTTP transport protocol
      when eAPI is present in the running-config. When the value is set to True, the
      HTTP protocol is enabled and when the value is set to False, the HTTP protocol
      is disabled. By default, when eAPI is first configured, the HTTP protocol is
      disabled.
    type: bool
    aliases:
    - enable_http
  http_port:
    description:
    - Configures the HTTP port that will listen for connections when the HTTP transport
      protocol is enabled.  This argument accepts integer values in the valid range
      of 1 to 65535.
    type: int
  https:
    description:
    - The C(https) argument controls the operating state of the HTTPS transport protocol
      when eAPI is present in the running-config. When the value is set to True, the
      HTTPS protocol is enabled and when the value is set to False, the HTTPS protocol
      is disabled. By default, when eAPI is first configured, the HTTPS protocol is
      enabled.
    type: bool
    aliases:
    - enable_https
  https_port:
    description:
    - Configures the HTTP port that will listen for connections when the HTTP transport
      protocol is enabled.  This argument accepts integer values in the valid range
      of 1 to 65535.
    type: int
  local_http:
    description:
    - The C(local_http) argument controls the operating state of the local HTTP transport
      protocol when eAPI is present in the running-config.  When the value is set
      to True, the HTTP protocol is enabled and restricted to connections from localhost
      only.  When the value is set to False, the HTTP local protocol is disabled.
    - Note is value is independent of the C(http) argument
    type: bool
    aliases:
    - enable_local_http
  local_http_port:
    description:
    - Configures the HTTP port that will listen for connections when the HTTP transport
      protocol is enabled.  This argument accepts integer values in the valid range
      of 1 to 65535.
    type: int
  socket:
    description:
    - The C(socket) argument controls the operating state of the UNIX Domain Socket
      used to receive eAPI requests.  When the value of this argument is set to True,
      the UDS will listen for eAPI requests.  When the value is set to False, the
      UDS will not be available to handle requests.  By default when eAPI is first
      configured, the UDS is disabled.
    type: bool
    aliases:
    - enable_socket
  timeout:
    description:
    - The time (in seconds) to wait for the eAPI configuration to be reflected in
      the running-config.
    type: int
    default: 30
  vrf:
    description:
    - The C(vrf) argument will configure eAPI to listen for connections in the specified
      VRF.  By default, eAPI transports will listen for connections in the global
      table.  This value requires the VRF to already be created otherwise the task
      will fail.
    default: default
    type: str
  config:
    description:
    - The module, by default, will connect to the remote device and retrieve the current
      running-config to use as a base for comparing against the contents of source.  There
      are times when it is not desirable to have the task get the current running-config
      for every task in a playbook.  The I(config) argument allows the implementer
      to pass in the configuration to use as the base config for comparison.
    type: str
  state:
    description:
    - The C(state) argument controls the operational state of eAPI on the remote device.  When
      this argument is set to C(started), eAPI is enabled to receive requests and
      when this argument is C(stopped), eAPI is disabled and will not receive requests.
    type: str
    default: started
    choices:
    - started
    - stopped
"""

EXAMPLES = """
- name: Enable eAPI access with default configuration
  arista.eos.eos_eapi:
    state: started

- name: Enable eAPI with no HTTP, HTTPS at port 9443, local HTTP at port 80, and socket
    enabled
  arista.eos.eos_eapi:
    state: started
    http: false
    https_port: 9443
    local_http: true
    local_http_port: 80
    socket: true

- name: Shutdown eAPI access
  arista.eos.eos_eapi:
    state: stopped
"""

RETURN = """
commands:
  description: The list of configuration mode commands to send to the device
  returned: always
  type: list
  sample:
    - management api http-commands
    - protocol http port 81
    - no protocol https
urls:
  description: Hash of URL endpoints eAPI is listening on per interface
  returned: when eAPI is started
  type: dict
  sample: {'Management1': ['http://172.26.10.1:80']}
session_name:
  description: The EOS config session name used to load the configuration
  returned: when changed is True
  type: str
  sample: ansible_1479315771
"""
import re
import time

from ansible.module_utils.basic import AnsibleModule

from ansible_collections.arista.eos.plugins.module_utils.network.eos.eos import (
    load_config,
    run_commands,
)


def validate_http_port(value, module):
    if not 1 <= value <= 65535:
        module.fail_json(msg="http_port must be between 1 and 65535")


def validate_https_port(value, module):
    if not 1 <= value <= 65535:
        module.fail_json(msg="http_port must be between 1 and 65535")


def validate_local_http_port(value, module):
    if not 1 <= value <= 65535:
        module.fail_json(msg="http_port must be between 1 and 65535")


def validate_vrf(value, module):
    out = run_commands(module, ["show vrf"])
    configured_vrfs = []
    lines = out[0].strip().splitlines()[3:]
    for line in lines:
        if not line:
            continue
        splitted_line = re.split(r"\s{2,}", line.strip())
        if len(splitted_line) > 2:
            configured_vrfs.append(splitted_line[0])

    configured_vrfs.append("default")
    if value not in configured_vrfs:
        module.fail_json(
            msg="vrf `%s` is not configured on the system" % value,
        )


def map_obj_to_commands(updates, module, warnings):
    commands = list()
    want, have = updates

    def needs_update(x):
        return want.get(x) is not None and (want.get(x) != have.get(x))

    def add(cmd):
        if "management api http-commands" not in commands:
            commands.insert(0, "management api http-commands")
        commands.append(cmd)

    if any((needs_update("http"), needs_update("http_port"))):
        if want["http"] is False:
            add("no protocol http")
        else:
            if have["http"] is False and want["http"] in (False, None):
                warnings.append(
                    "protocol http is not enabled, not configuring http port value",
                )
            else:
                port = want["http_port"] or 80
                add("protocol http port %s" % port)

    if any((needs_update("https"), needs_update("https_port"))):
        if want["https"] is False:
            add("no protocol https")
        else:
            if have["https"] is False and want["https"] in (False, None):
                warnings.append(
                    "protocol https is not enabled, not configuring https port value",
                )
            else:
                port = want["https_port"] or 443
                add("protocol https port %s" % port)

    if any((needs_update("local_http"), needs_update("local_http_port"))):
        if want["local_http"] is False:
            add("no protocol http localhost")
        else:
            if have["local_http"] is False and want["local_http"] in (
                False,
                None,
            ):
                warnings.append(
                    "protocol local_http is not enabled, not configuring local_http port value",
                )
            else:
                port = want["local_http_port"] or 8080
                add("protocol http localhost port %s" % port)

    if any((needs_update("socket"), needs_update("socket"))):
        if want["socket"] is False:
            add("no protocol unix-socket")
        else:
            add("protocol unix-socket")
    if needs_update("state"):
        if want["state"] == "stopped":
            add("shutdown")
        elif want["state"] == "started":
            add("no shutdown")

    if needs_update("vrf"):
        add("vrf %s" % want["vrf"])
        # switching operational vrfs here
        # need to add the desired state as well
        if want["state"] == "stopped":
            add("shutdown")
        elif want["state"] == "started":
            add("no shutdown")

    return commands


def parse_state(data):
    if data[0]["enabled"]:
        return "started"
    else:
        return "stopped"


def map_config_to_obj(module):
    out = run_commands(module, ["show management api http-commands | json"])
    return {
        "http": out[0]["httpServer"]["configured"],
        "http_port": out[0]["httpServer"]["port"],
        "https": out[0]["httpsServer"]["configured"],
        "https_port": out[0]["httpsServer"]["port"],
        "local_http": out[0]["localHttpServer"]["configured"],
        "local_http_port": out[0]["localHttpServer"]["port"],
        "socket": out[0]["unixSocketServer"]["configured"],
        "vrf": out[0]["vrf"] or "default",
        "state": parse_state(out),
    }


def map_params_to_obj(module):
    obj = {
        "http": module.params["http"],
        "http_port": module.params["http_port"],
        "https": module.params["https"],
        "https_port": module.params["https_port"],
        "local_http": module.params["local_http"],
        "local_http_port": module.params["local_http_port"],
        "socket": module.params["socket"],
        "vrf": module.params["vrf"],
        "state": module.params["state"],
    }

    for key, value in obj.items():
        if value:
            validator = globals().get("validate_%s" % key)
            if validator:
                validator(value, module)

    return obj


def verify_state(updates, module):
    want, have = updates

    invalid_state = [
        ("http", "httpServer"),
        ("https", "httpsServer"),
        ("local_http", "localHttpServer"),
        ("socket", "unixSocketServer"),
    ]

    timeout = module.params["timeout"]
    state = module.params["state"]

    while invalid_state:
        out = run_commands(
            module,
            ["show management api http-commands | json"],
        )
        for index, item in enumerate(invalid_state):
            want_key, eapi_key = item
            if want[want_key] is not None:
                if want[want_key] == out[0][eapi_key]["running"]:
                    del invalid_state[index]
            elif state == "stopped":
                if not out[0][eapi_key]["running"]:
                    del invalid_state[index]
            else:
                del invalid_state[index]
        time.sleep(1)
        timeout -= 1
        if timeout == 0:
            module.fail_json(
                msg="timeout expired before eapi running state changed",
            )


def collect_facts(module, result):
    out = run_commands(module, ["show management api http-commands | json"])
    facts = dict(eos_eapi_urls=dict())
    for each in out[0]["urls"]:
        intf, url = each.split(":", 1)
        key = str(intf).strip()
        if key not in facts["eos_eapi_urls"]:
            facts["eos_eapi_urls"][key] = list()
        facts["eos_eapi_urls"][key].append(str(url).strip())
    result["ansible_facts"] = facts


def main():
    """main entry point for module execution"""
    argument_spec = dict(
        http=dict(aliases=["enable_http"], type="bool"),
        http_port=dict(type="int"),
        https=dict(aliases=["enable_https"], type="bool"),
        https_port=dict(type="int"),
        local_http=dict(aliases=["enable_local_http"], type="bool"),
        local_http_port=dict(type="int"),
        socket=dict(aliases=["enable_socket"], type="bool"),
        timeout=dict(type="int", default=30),
        vrf=dict(default="default"),
        config=dict(),
        state=dict(default="started", choices=["stopped", "started"]),
    )

    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
    )

    result = {"changed": False}

    warnings = list()
    if module.params["config"]:
        warnings.append(
            "config parameter is no longer necessary and will be ignored",
        )

    want = map_params_to_obj(module)
    have = map_config_to_obj(module)

    commands = map_obj_to_commands((want, have), module, warnings)
    result["commands"] = commands

    if commands:
        commit = not module.check_mode
        response = load_config(module, commands, commit=commit)
        if response.get("diff") and module._diff:
            result["diff"] = {"prepared": response.get("diff")}
        result["session_name"] = response.get("session")
        result["changed"] = True

    if result["changed"]:
        verify_state((want, have), module)

    collect_facts(module, result)

    if warnings:
        result["warnings"] = warnings

    module.exit_json(**result)


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
# Copyright 2021 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function


__metaclass__ = type

DOCUMENTATION = """
    author: Katherine Case (@Qalthos)
    name: config
    short_description: Define configurable options for configuration validate plugin
    description:
    - This sub plugin documentation provides the configurable options that can be passed
      to the validate plugins when C(ansible.utils.config) is used as a value for
      engine option.
    version_added: 2.1.0
    notes:
    - The value of I(data) option should be a candidate device configuration.
    - The value of I(criteria) should be a B(list) of rules the candidate configuration
      will be checked against, or a yaml document containing those rules.
"""


EXAMPLES = r"""
- name: Interface description should not be more than 8 chars
  example: "Matches description this-is-a-long-description"
  rule: 'description\s(.{9,})'
  action: warn

- name: Ethernet interface names should be in format Ethernet[Slot/chassis number].[sub-intf number (optional)]
  example: "Matches interface Eth1/1, interface Eth 1/1, interface Ethernet 1/1, interface Ethernet 1/1.100"
  rule: 'interface\s[eE](?!\w{7}\d/\d(.\d+)?)'
  action: fail

- name: Loopback interface names should be in format loopback[Virtual Interface Number]
  example: "Matches interface Lo10, interface Loopback 10"
  rule: 'interface\s[lL](?!\w{7}\d)'
  action: fail

- name: Port Channel names should be in format port-channel[Port Channel number].[sub-intf number (optional)]
  example: "Matches interface port-channel 10, interface po10, interface port-channel 10.1"
  rule: 'interface\s[pP](?!\w{3}-\w{7}\d(.\d+)?)'
  action: fail
"""

import re

from io import StringIO

from ansible.errors import AnsibleError
from ansible.module_utils.common.text.converters import to_text

from ansible_collections.ansible.utils.plugins.module_utils.common.utils import to_list
from ansible_collections.ansible.utils.plugins.plugin_utils.base.validate import ValidateBase


try:
    import yaml

    # use C version if possible for speedup
    try:
        from yaml import CSafeLoader as SafeLoader
    except ImportError:
        from yaml import SafeLoader
    HAS_YAML = True
except ImportError:
    HAS_YAML = False


def format_message(match, line_number, criteria):
    """Format warning or error message based on given line and criteria."""
    return 'At line {line_number}: {message}\nFound "{line}"'.format(
        line_number=line_number + 1,
        message=criteria["name"],
        line=match.string,
    )


class Validate(ValidateBase):
    def _check_args(self):
        """Ensure specific args are set

        :return: None: In case all arguments passed are valid
        """

        try:
            if isinstance(self._criteria, str):
                self._criteria = yaml.load(StringIO(self._criteria), Loader=SafeLoader)
        except yaml.parser.ParserError as exc:
            msg = (
                "'criteria' option value is invalid, value should be valid YAML."
                " Failed to read with error '{err}'".format(
                    err=to_text(exc, errors="surrogate_then_replace"),
                )
            )
            raise AnsibleError(msg)

        issues = []
        for item in to_list(self._criteria):
            if "name" not in item:
                issues.append('Criteria {item} missing "name" key'.format(item=item))
            if "action" not in item:
                issues.append('Criteria {item} missing "action" key'.format(item=item))
            elif item["action"] not in ("warn", "fail"):
                issues.append(
                    'Action in criteria {item} is not one of "warn" or "fail"'.format(
                        item=item,
                    ),
                )
            if "rule" not in item:
                issues.append('Criteria {item} missing "rule" key'.format(item=item))
            else:
                try:
                    item["rule"] = re.compile(item["rule"])
                except re.error as exc:
                    issues.append(
                        'Failed to compile regex "{rule}": {exc}'.format(
                            rule=item["rule"],
                            exc=exc,
                        ),
                    )

        if issues:
            msg = "\n".join(issues)
            raise AnsibleError(msg)

    def validate(self):
        """Std entry point for a validate execution

        :return: Errors or parsed text as structured data
        :rtype: dict

        :example:

        The parse function of a parser should return a dict:
        {"errors": [a list of errors]}
        or
        {"parsed": obj}
        """
        self._check_args()

        try:
            self._validate_config()
        except Exception as exc:
            return {"errors": to_text(exc, errors="surrogate_then_replace")}

        return self._result

    def _validate_config(self):
        warnings = []
        errors = []
        error_messages = []

        for criteria in self._criteria:
            for line_number, line in enumerate(self._data.split("\n")):
                match = criteria["rule"].search(line)
                if match:
                    if criteria["action"] == "warn":
                        warnings.append(format_message(match, line_number, criteria))
                    if criteria["action"] == "fail":
                        errors.append({"message": criteria["name"], "found": line})
                        error_messages.append(
                            format_message(match, line_number, criteria),
                        )

        if errors:
            if "errors" not in self._result:
                self._result["errors"] = []
            self._result["errors"].extend(errors)
        if error_messages:
            if "msg" not in self._result:
                self._result["msg"] = "\n".join(error_messages)
            else:
                self._result["msg"] += "\n".join(error_messages)
        if warnings:
            if "warnings" not in self._result:
                self._result["warnings"] = []
            self._result["warnings"].extend(warnings)



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or
# https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
module: networks_group_policies
short_description: Resource module for networks _group _policies
description:
  - Manage operations create, update and delete of the resource networks _group _policies.
  - Create a group policy.
  - Delete a group policy.
  - Update a group policy.
version_added: '1.0.0'
extends_documentation_fragment:
  - cisco.meraki.module
author: Francisco Munoz (@fmunoz)
options:
  bandwidth:
    description: The bandwidth settings for clients bound to your group policy.
    suboptions:
      bandwidthLimits:
        description: The bandwidth limits object, specifying upload and download speed
          for clients bound to the group policy. These are only enforced if 'settings'
          is set to 'custom'.
        suboptions:
          limitDown:
            description: The maximum download limit (integer, in Kbps). Null indicates
              no limit.
            type: int
          limitUp:
            description: The maximum upload limit (integer, in Kbps). Null indicates
              no limit.
            type: int
        type: dict
      settings:
        description: How bandwidth limits are enforced. Can be 'network default',
          'ignore' or 'custom'.
        type: str
    type: dict
  bonjourForwarding:
    description: The Bonjour settings for your group policy. Only valid if your network
      has a wireless configuration.
    suboptions:
      rules:
        description: A list of the Bonjour forwarding rules for your group policy.
          If 'settings' is set to 'custom', at least one rule must be specified.
        elements: dict
        suboptions:
          description:
            description: A description for your Bonjour forwarding rule. Optional.
            type: str
          services:
            description: A list of Bonjour services. At least one service must be
              specified. Available services are 'All Services', 'AFP', 'AirPlay',
              'Apple screen share', 'BitTorrent', 'Chromecast', 'FTP', 'iChat', 'iTunes',
              'Printers', 'Samba', 'Scanners', 'Spotify' and 'SSH'.
            elements: str
            type: list
          vlanId:
            description: The ID of the service VLAN. Required.
            type: str
        type: list
      settings:
        description: How Bonjour rules are applied. Can be 'network default', 'ignore'
          or 'custom'.
        type: str
    type: dict
  contentFiltering:
    description: The content filtering settings for your group policy.
    suboptions:
      allowedUrlPatterns:
        description: Settings for allowed URL patterns.
        suboptions:
          patterns:
            description: A list of URL patterns that are allowed.
            elements: str
            type: list
          settings:
            description: How URL patterns are applied. Can be 'network default', 'append'
              or 'override'.
            type: str
        type: dict
      blockedUrlCategories:
        description: Settings for blocked URL categories.
        suboptions:
          categories:
            description: A list of URL categories to block.
            elements: str
            type: list
          settings:
            description: How URL categories are applied. Can be 'network default',
              'append' or 'override'.
            type: str
        type: dict
      blockedUrlPatterns:
        description: Settings for blocked URL patterns.
        suboptions:
          patterns:
            description: A list of URL patterns that are blocked.
            elements: str
            type: list
          settings:
            description: How URL patterns are applied. Can be 'network default', 'append'
              or 'override'.
            type: str
        type: dict
    type: dict
  firewallAndTrafficShaping:
    description: The firewall and traffic shaping rules and settings for your policy.
    suboptions:
      l3FirewallRules:
        description: An ordered array of the L3 firewall rules.
        elements: dict
        suboptions:
          comment:
            description: Description of the rule (optional).
            type: str
          destCidr:
            description: Destination IP address (in IP or CIDR notation), a fully-qualified
              domain name (FQDN, if your network supports it) or 'any'.
            type: str
          destPort:
            description: Destination port (integer in the range 1-65535), a port range
              (e.g. 8080-9090), or 'any'.
            type: str
          policy:
            description: '''allow'' or ''deny'' traffic specified by this rule.'
            type: str
          protocol:
            description: The type of protocol (must be 'tcp', 'udp', 'icmp', 'icmp6'
              or 'any').
            type: str
        type: list
      l7FirewallRules:
        description: An ordered array of L7 firewall rules.
        elements: dict
        suboptions:
          policy:
            description: The policy applied to matching traffic. Must be 'deny'.
            type: str
          type:
            description: Type of the L7 Rule. Must be 'application', 'applicationCategory',
              'host', 'port' or 'ipRange'.
            type: str
          value:
            description: The 'value' of what you want to block. If 'type' is 'host',
              'port' or 'ipRange', 'value' must be a string matching either a hostname
              (e.g. Somewhere.com), a port (e.g. 8080), or an IP range (e.g. 192.1.0.0/16).
              If 'type' is 'application' or 'applicationCategory', then 'value' must
              be an object with an ID for the application.
            type: str
        type: list
      settings:
        description: How firewall and traffic shaping rules are enforced. Can be 'network
          default', 'ignore' or 'custom'.
        type: str
      trafficShapingRules:
        description: An array of traffic shaping rules. Rules are applied in the order
          that they are specified in. An empty list (or null) means no rules. Note
          that you are allowed a maximum of 8 rules.
        elements: dict
        suboptions:
          definitions:
            description: A list of objects describing the definitions of your traffic
              shaping rule. At least one definition is required.
            elements: dict
            suboptions:
              type:
                description: The type of definition. Can be one of 'application',
                  'applicationCategory', 'host', 'port', 'ipRange' or 'localNet'.
                type: str
              value:
                description: If "type" is 'host', 'port', 'ipRange' or 'localNet',
                  then "value" must be a string, matching either a hostname (e.g.
                  "somesite.com"), a port (e.g. 8080), or an IP range ("192.1.0.0",
                  "192.1.0.0/16", or "10.1.0.0/16 80"). 'localNet' also supports CIDR
                  notation, excluding custom ports. If "type" is 'application' or
                  'applicationCategory', then "value" must be an object with the structure
                  { "id" "meraki layer7/..." }, where "id" is the application category
                  or application ID (for a list of IDs for your network, use the trafficShaping/applicationCategories
                  endpoint).
                type: str
            type: list
          dscpTagValue:
            description: The DSCP tag applied by your rule. Null means 'Do not change
              DSCP tag'. For a list of possible tag values, use the trafficShaping/dscpTaggingOptions
              endpoint.
            type: int
          pcpTagValue:
            description: The PCP tag applied by your rule. Can be 0 (lowest priority)
              through 7 (highest priority). Null means 'Do not set PCP tag'.
            type: int
          perClientBandwidthLimits:
            description: An object describing the bandwidth settings for your rule.
            suboptions:
              bandwidthLimits:
                description: The bandwidth limits object, specifying the upload ('limitUp')
                  and download ('limitDown') speed in Kbps. These are only enforced
                  if 'settings' is set to 'custom'.
                suboptions:
                  limitDown:
                    description: The maximum download limit (integer, in Kbps).
                    type: int
                  limitUp:
                    description: The maximum upload limit (integer, in Kbps).
                    type: int
                type: dict
              settings:
                description: How bandwidth limits are applied by your rule. Can be
                  one of 'network default', 'ignore' or 'custom'.
                type: str
            type: dict
          priority:
            description: A string, indicating the priority level for packets bound
              to your rule. Can be 'low', 'normal' or 'high'.
            type: str
        type: list
    type: dict
  force:
    description: Force query parameter. If true, the system deletes the GP even if
      there are active clients using the GP. After deletion, active clients that were
      assigned to that Group Policy will be left without any policy applied. Default
      is false.
    type: bool
  groupPolicyId:
    description: GroupPolicyId path parameter. Group policy ID.
    type: str
  name:
    description: The name for your group policy. Required.
    type: str
  networkId:
    description: NetworkId path parameter. Network ID.
    type: str
  scheduling:
    description: The schedule for the group policy. Schedules are applied to days
      of the week.
    suboptions:
      enabled:
        description: Whether scheduling is enabled (true) or disabled (false). Defaults
          to false. If true, the schedule objects for each day of the week (monday
          - sunday) are parsed.
        type: bool
      friday:
        description: The schedule object for Friday.
        suboptions:
          active:
            description: Whether the schedule is active (true) or inactive (false)
              during the time specified between 'from' and 'to'. Defaults to true.
            type: bool
          from:
            description: The time, from '00 00' to '24 00'. Must be less than the
              time specified in 'to'. Defaults to '00 00'. Only 30 minute increments
              are allowed.
            type: str
          to:
            description: The time, from '00 00' to '24 00'. Must be greater than the
              time specified in 'from'. Defaults to '24 00'. Only 30 minute increments
              are allowed.
            type: str
        type: dict
      monday:
        description: The schedule object for Monday.
        suboptions:
          active:
            description: Whether the schedule is active (true) or inactive (false)
              during the time specified between 'from' and 'to'. Defaults to true.
            type: bool
          from:
            description: The time, from '00 00' to '24 00'. Must be less than the
              time specified in 'to'. Defaults to '00 00'. Only 30 minute increments
              are allowed.
            type: str
          to:
            description: The time, from '00 00' to '24 00'. Must be greater than the
              time specified in 'from'. Defaults to '24 00'. Only 30 minute increments
              are allowed.
            type: str
        type: dict
      saturday:
        description: The schedule object for Saturday.
        suboptions:
          active:
            description: Whether the schedule is active (true) or inactive (false)
              during the time specified between 'from' and 'to'. Defaults to true.
            type: bool
          from:
            description: The time, from '00 00' to '24 00'. Must be less than the
              time specified in 'to'. Defaults to '00 00'. Only 30 minute increments
              are allowed.
            type: str
          to:
            description: The time, from '00 00' to '24 00'. Must be greater than the
              time specified in 'from'. Defaults to '24 00'. Only 30 minute increments
              are allowed.
            type: str
        type: dict
      sunday:
        description: The schedule object for Sunday.
        suboptions:
          active:
            description: Whether the schedule is active (true) or inactive (false)
              during the time specified between 'from' and 'to'. Defaults to true.
            type: bool
          from:
            description: The time, from '00 00' to '24 00'. Must be less than the
              time specified in 'to'. Defaults to '00 00'. Only 30 minute increments
              are allowed.
            type: str
          to:
            description: The time, from '00 00' to '24 00'. Must be greater than the
              time specified in 'from'. Defaults to '24 00'. Only 30 minute increments
              are allowed.
            type: str
        type: dict
      thursday:
        description: The schedule object for Thursday.
        suboptions:
          active:
            description: Whether the schedule is active (true) or inactive (false)
              during the time specified between 'from' and 'to'. Defaults to true.
            type: bool
          from:
            description: The time, from '00 00' to '24 00'. Must be less than the
              time specified in 'to'. Defaults to '00 00'. Only 30 minute increments
              are allowed.
            type: str
          to:
            description: The time, from '00 00' to '24 00'. Must be greater than the
              time specified in 'from'. Defaults to '24 00'. Only 30 minute increments
              are allowed.
            type: str
        type: dict
      tuesday:
        description: The schedule object for Tuesday.
        suboptions:
          active:
            description: Whether the schedule is active (true) or inactive (false)
              during the time specified between 'from' and 'to'. Defaults to true.
            type: bool
          from:
            description: The time, from '00 00' to '24 00'. Must be less than the
              time specified in 'to'. Defaults to '00 00'. Only 30 minute increments
              are allowed.
            type: str
          to:
            description: The time, from '00 00' to '24 00'. Must be greater than the
              time specified in 'from'. Defaults to '24 00'. Only 30 minute increments
              are allowed.
            type: str
        type: dict
      wednesday:
        description: The schedule object for Wednesday.
        suboptions:
          active:
            description: Whether the schedule is active (true) or inactive (false)
              during the time specified between 'from' and 'to'. Defaults to true.
            type: bool
          from:
            description: The time, from '00 00' to '24 00'. Must be less than the
              time specified in 'to'. Defaults to '00 00'. Only 30 minute increments
              are allowed.
            type: str
          to:
            description: The time, from '00 00' to '24 00'. Must be greater than the
              time specified in 'from'. Defaults to '24 00'. Only 30 minute increments
              are allowed.
            type: str
        type: dict
    type: dict
  splashAuthSettings:
    description: Whether clients bound to your policy will bypass splash authorization
      or behave according to the network's rules. Can be one of 'network default'
      or 'bypass'. Only available if your network has a wireless configuration.
    type: str
  vlanTagging:
    description: The VLAN tagging settings for your group policy. Only available if
      your network has a wireless configuration.
    suboptions:
      settings:
        description: How VLAN tagging is applied. Can be 'network default', 'ignore'
          or 'custom'.
        type: str
      vlanId:
        description: The ID of the vlan you want to tag. This only applies if 'settings'
          is set to 'custom'.
        type: str
    type: dict
requirements:
  - meraki >= 2.4.9
  - python >= 3.5
seealso:
  - name: Cisco Meraki documentation for networks createNetworkGroupPolicy
    description: Complete reference of the createNetworkGroupPolicy API.
    link: https://developer.cisco.com/meraki/api-v1/#!create-network-group-policy
  - name: Cisco Meraki documentation for networks deleteNetworkGroupPolicy
    description: Complete reference of the deleteNetworkGroupPolicy API.
    link: https://developer.cisco.com/meraki/api-v1/#!delete-network-group-policy
  - name: Cisco Meraki documentation for networks updateNetworkGroupPolicy
    description: Complete reference of the updateNetworkGroupPolicy API.
    link: https://developer.cisco.com/meraki/api-v1/#!update-network-group-policy
notes:
  - SDK Method used are
    networks.Networks.create_network_group_policy,
    networks.Networks.delete_network_group_policy,
    networks.Networks.update_network_group_policy,
  - Paths used are
    post /networks/{networkId}/groupPolicies,
    delete /networks/{networkId}/groupPolicies/{groupPolicyId},
    put /networks/{networkId}/groupPolicies/{groupPolicyId},
"""

EXAMPLES = r"""
- name: Create
  cisco.meraki.networks_group_policies:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    state: present
    bandwidth:
      bandwidthLimits:
        limitDown: 1000000
        limitUp: 1000000
      settings: custom
    bonjourForwarding:
      rules:
        - description: A simple bonjour rule
          services:
            - All Services
          vlanId: '1'
      settings: custom
    contentFiltering:
      allowedUrlPatterns:
        patterns: []
        settings: network default
      blockedUrlCategories:
        categories:
          - meraki:contentFiltering/category/1
          - meraki:contentFiltering/category/7
        settings: override
      blockedUrlPatterns:
        patterns:
          - http://www.example.com
          - http://www.betting.com
        settings: append
    firewallAndTrafficShaping:
      l3FirewallRules:
        - comment: Allow TCP traffic to subnet with HTTP servers.
          destCidr: 192.168.1.0/24
          destPort: '443'
          policy: allow
          protocol: tcp
      l7FirewallRules:
        - policy: deny
          type: host
          value: google.com
      settings: custom
      trafficShapingRules:
        - definitions:
            - type: host
              value: google.com
          dscpTagValue: 0
          pcpTagValue: 0
          perClientBandwidthLimits:
            bandwidthLimits:
              limitDown: 1000000
              limitUp: 1000000
            settings: custom
          priority: normal
    name: No video streaming
    networkId: string
    scheduling:
      enabled: true
      friday:
        active: true
        from: '9:00'
        to: '17:00'
      monday:
        active: true
        from: '9:00'
        to: '17:00'
      saturday:
        active: true
        from: '9:00'
        to: '17:00'
      sunday:
        active: true
        from: '9:00'
        to: '17:00'
      thursday:
        active: true
        from: '9:00'
        to: '17:00'
      tuesday:
        active: true
        from: '9:00'
        to: '17:00'
      wednesday:
        active: true
        from: '9:00'
        to: '17:00'
    splashAuthSettings: bypass
    vlanTagging:
      settings: custom
      vlanId: '1'
- name: Update by id
  cisco.meraki.networks_group_policies:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    state: present
    bandwidth:
      bandwidthLimits:
        limitDown: 1000000
        limitUp: 1000000
      settings: custom
    bonjourForwarding:
      rules:
        - description: A simple bonjour rule
          services:
            - All Services
          vlanId: '1'
      settings: custom
    contentFiltering:
      allowedUrlPatterns:
        patterns: []
        settings: network default
      blockedUrlCategories:
        categories:
          - meraki:contentFiltering/category/1
          - meraki:contentFiltering/category/7
        settings: override
      blockedUrlPatterns:
        patterns:
          - http://www.example.com
          - http://www.betting.com
        settings: append
    firewallAndTrafficShaping:
      l3FirewallRules:
        - comment: Allow TCP traffic to subnet with HTTP servers.
          destCidr: 192.168.1.0/24
          destPort: '443'
          policy: allow
          protocol: tcp
      l7FirewallRules:
        - policy: deny
          type: host
          value: google.com
      settings: custom
      trafficShapingRules:
        - definitions:
            - type: host
              value: google.com
          dscpTagValue: 0
          pcpTagValue: 0
          perClientBandwidthLimits:
            bandwidthLimits:
              limitDown: 1000000
              limitUp: 1000000
            settings: custom
          priority: normal
    groupPolicyId: string
    name: No video streaming
    networkId: string
    scheduling:
      enabled: true
      friday:
        active: true
        from: '9:00'
        to: '17:00'
      monday:
        active: true
        from: '9:00'
        to: '17:00'
      saturday:
        active: true
        from: '9:00'
        to: '17:00'
      sunday:
        active: true
        from: '9:00'
        to: '17:00'
      thursday:
        active: true
        from: '9:00'
        to: '17:00'
      tuesday:
        active: true
        from: '9:00'
        to: '17:00'
      wednesday:
        active: true
        from: '9:00'
        to: '17:00'
    splashAuthSettings: bypass
    vlanTagging:
      settings: custom
      vlanId: '1'
- name: Delete by id
  cisco.meraki.networks_group_policies:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    state: absent
    force: true
    groupPolicyId: string
    networkId: string
"""
RETURN = r"""
meraki_response:
  description: A dictionary or list with the response returned by the Cisco Meraki Python SDK
  returned: always
  type: dict
  sample: >
    {
      "groupPolicyId": "string",
      "scheduling": {
        "enabled": true,
        "monday": {
          "active": true,
          "from": "string",
          "to": "string"
        },
        "tuesday": {
          "active": true,
          "from": "string",
          "to": "string"
        },
        "wednesday": {
          "active": true,
          "from": "string",
          "to": "string"
        },
        "thursday": {
          "active": true,
          "from": "string",
          "to": "string"
        },
        "friday": {
          "active": true,
          "from": "string",
          "to": "string"
        },
        "saturday": {
          "active": true,
          "from": "string",
          "to": "string"
        },
        "sunday": {
          "active": true,
          "from": "string",
          "to": "string"
        }
      },
      "bandwidth": {
        "settings": "string",
        "bandwidthLimits": {
          "limitUp": 0,
          "limitDown": 0
        }
      },
      "firewallAndTrafficShaping": {
        "settings": "string",
        "trafficShapingRules": [
          {
            "definitions": [
              {
                "type": "string",
                "value": "string"
              }
            ],
            "perClientBandwidthLimits": {
              "settings": "string",
              "bandwidthLimits": {
                "limitUp": 0,
                "limitDown": 0
              }
            },
            "dscpTagValue": 0,
            "pcpTagValue": 0,
            "priority": "string"
          }
        ],
        "l3FirewallRules": [
          {
            "comment": "string",
            "policy": "string",
            "protocol": "string",
            "destPort": "string",
            "destCidr": "string"
          }
        ],
        "l7FirewallRules": [
          {
            "policy": "string",
            "type": "string",
            "value": "string"
          }
        ]
      },
      "contentFiltering": {
        "allowedUrlPatterns": {
          "settings": "string",
          "patterns": [
            "string"
          ]
        },
        "blockedUrlPatterns": {
          "settings": "string",
          "patterns": [
            "string"
          ]
        },
        "blockedUrlCategories": {
          "settings": "string",
          "categories": [
            "string"
          ]
        }
      },
      "splashAuthSettings": "string",
      "vlanTagging": {
        "settings": "string",
        "vlanId": "string"
      },
      "bonjourForwarding": {
        "settings": "string",
        "rules": [
          {
            "description": "string",
            "vlanId": "string",
            "services": [
              "string"
            ]
          }
        ]
      }
    }
"""



# Copyright (c) 2016, Peter Sagerson <psagers@ignorare.net>
# Copyright (c) 2016, Jiri Tyr <jiri.tyr@gmail.com>
# Copyright (c) 2017-2018 Keller Fuchs (@KellerFuchs) <kellerfuchs@hashbang.sh>
#
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

# Note that this module util is **PRIVATE** to the collection. It can have breaking changes at any time.
# Do not use this from other collections or standalone plugins/modules!

from __future__ import annotations

import re
import traceback
import typing as t

if t.TYPE_CHECKING:
    from ansible.module_utils.basic import AnsibleModule

try:
    import ldap
    import ldap.dn
    import ldap.filter
    import ldap.sasl

    HAS_LDAP = True

    SASCL_CLASS = {
        "gssapi": ldap.sasl.gssapi,
        "external": ldap.sasl.external,
    }
except ImportError:
    HAS_LDAP = False


def gen_specs(**specs: t.Any) -> dict[str, t.Any]:
    specs.update(
        {
            "bind_dn": dict(),
            "bind_pw": dict(default="", no_log=True),
            "ca_path": dict(type="path"),
            "dn": dict(required=True),
            "referrals_chasing": dict(type="str", default="anonymous", choices=["disabled", "anonymous"]),
            "server_uri": dict(default="ldapi:///"),
            "start_tls": dict(default=False, type="bool"),
            "validate_certs": dict(default=True, type="bool"),
            "sasl_class": dict(choices=["external", "gssapi"], default="external", type="str"),
            "xorder_discovery": dict(choices=["enable", "auto", "disable"], default="auto", type="str"),
            "client_cert": dict(default=None, type="path"),
            "client_key": dict(default=None, type="path"),
        }
    )

    return specs


def ldap_required_together() -> list[list[str]]:
    return [["client_cert", "client_key"]]


class LdapGeneric:
    def __init__(self, module: AnsibleModule) -> None:
        # Shortcuts
        self.module = module
        self.bind_dn = self.module.params["bind_dn"]
        self.bind_pw = self.module.params["bind_pw"]
        self.ca_path = self.module.params["ca_path"]
        self.referrals_chasing = self.module.params["referrals_chasing"]
        self.server_uri = self.module.params["server_uri"]
        self.start_tls = self.module.params["start_tls"]
        self.verify_cert = self.module.params["validate_certs"]
        self.sasl_class = self.module.params["sasl_class"]
        self.xorder_discovery = self.module.params["xorder_discovery"]
        self.client_cert = self.module.params["client_cert"]
        self.client_key = self.module.params["client_key"]

        # Establish connection
        self.connection = self._connect_to_ldap()

        if self.xorder_discovery == "enable" or (self.xorder_discovery == "auto" and not self._xorder_dn()):
            # Try to find the X_ORDERed version of the DN
            self.dn = self._find_dn()
        else:
            self.dn = self.module.params["dn"]

    def fail(self, msg: str, exn: str | Exception) -> t.NoReturn:
        self.module.fail_json(msg=msg, details=f"{exn}", exception=traceback.format_exc())

    def _find_dn(self) -> str:
        dn: str = self.module.params["dn"]

        explode_dn = ldap.dn.explode_dn(dn)

        if len(explode_dn) > 1:
            try:
                escaped_value = ldap.filter.escape_filter_chars(explode_dn[0])
                filterstr = f"({escaped_value})"
                dns = self.connection.search_s(",".join(explode_dn[1:]), ldap.SCOPE_ONELEVEL, filterstr)
                if len(dns) == 1:
                    dn, dummy = dns[0]
            except Exception:
                pass

        return dn

    def _connect_to_ldap(self):
        if not self.verify_cert:
            ldap.set_option(ldap.OPT_X_TLS_REQUIRE_CERT, ldap.OPT_X_TLS_NEVER)

        if self.ca_path:
            ldap.set_option(ldap.OPT_X_TLS_CACERTFILE, self.ca_path)

        if self.client_cert and self.client_key:
            ldap.set_option(ldap.OPT_X_TLS_CERTFILE, self.client_cert)
            ldap.set_option(ldap.OPT_X_TLS_KEYFILE, self.client_key)

        connection = ldap.initialize(self.server_uri)

        if self.referrals_chasing == "disabled":
            # Switch off chasing of referrals (https://github.com/ansible-collections/community.general/issues/1067)
            connection.set_option(ldap.OPT_REFERRALS, 0)

        if self.start_tls:
            try:
                connection.start_tls_s()
            except ldap.LDAPError as e:
                self.fail("Cannot start TLS.", e)

        try:
            if self.bind_dn is not None:
                connection.simple_bind_s(self.bind_dn, self.bind_pw)
            else:
                klass = SASCL_CLASS.get(self.sasl_class, ldap.sasl.external)
                connection.sasl_interactive_bind_s("", klass())
        except ldap.LDAPError as e:
            self.fail("Cannot bind to the server.", e)

        return connection

    def _xorder_dn(self) -> bool:
        # match X_ORDERed DNs
        regex = r".+\{\d+\}.+"
        explode_dn = ldap.dn.explode_dn(self.module.params["dn"])

        return re.match(regex, explode_dn[0]) is not None



#!/usr/bin/python
# -*- coding: utf-8 -*-

# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['preview'],
                    'supported_by': 'community'}

DOCUMENTATION = r'''
---
module: intersight_uuid_pool
short_description: UUID Pool configuration for Cisco Intersight
description:
  - UUID Pool configuration for Cisco Intersight.
  - Used to configure UUID pools settings on Cisco Intersight managed devices.
  - For more information see L(Cisco Intersight,https://intersight.com/apidocs).
extends_documentation_fragment: intersight
options:
  state:
    description:
      - If C(present), will verify the resource is present and will create if needed.
      - If C(absent), will verify the resource is absent and will delete if needed.
    type: str
    choices: [present, absent]
    default: present
  organization:
    description:
      - The name of the Organization this resource is assigned to.
      - Profiles, Policies, and Pools that are created within a Custom Organization are applicable only to devices in the same Organization.
    type: str
    default: default
  name:
    description:
      - The name assigned to the UUID Pool.
      - The name must be between 1 and 64 alphanumeric characters, allowing special characters :-_.
    type: str
    required: true
  tags:
    description:
      - List of tags in Key:<user-defined key> Value:<user-defined value> format.
    type: list
    elements: dict
  description:
    description:
      - The user-defined description of the UUID Pool.
      - Description can contain letters(a-z, A-Z), numbers(0-9), hyphen(-), period(.), colon(:), or an underscore(_).
    type: str
    aliases: [descr]
  assignment_order:
    description:
      - Assignment order decides the order in which the next identifier is allocated.
      - If C(sequential), identifiers are assigned in a sequential order.
      - If C(default), the system determines the assignment order.
    type: str
    choices: [default, sequential]
    default: default
  prefix:
    description:
      - The UUID prefix used for all UUID suffix blocks.
      - UUID must be in uppercase in order to support idempotency.
      - The prefix must be in the format XXXXXXXX-XXXX-XXXX (8-4-4) where X is a hexadecimal character (0-9, A-F).
      - This prefix will be combined with the suffix blocks to form complete UUIDs.
      - Required for C(state=present)
    type: str
  uuid_suffix_blocks:
    description:
      - List of UUID suffix blocks that define the range of UUIDs available in the pool.
      - Each block defines a starting suffix and the number of consecutive UUIDs in that block.
      - Required for C(state=present)
    type: list
    elements: dict
    suboptions:
      from:
        description:
          - The starting UUID suffix for this block.
          - UUID must be in uppercase in order to support idempotency.
          - The suffix must be in the format XXXX-XXXXXXXXXXXX (4-12) where X is a hexadecimal character (0-9, A-F).
          - This suffix will be combined with the pool prefix to form complete UUIDs.
        type: str
      size:
        description:
          - The number of consecutive UUIDs in this block.
          - Must be a positive integer representing how many UUIDs are available starting from the 'from' suffix.
        type: int
author:
  - Shahar Golshani (@sgolshan)
'''

EXAMPLES = r'''
- name: Configure UUID Pool
  cisco.intersight.intersight_uuid_pool:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    organization: DevNet
    name: lab-uuid-pool
    description: UUID Pool for lab use
    assignment_order: sequential
    prefix: "550E8400-E29B-41D4"
    uuid_suffix_blocks:
      - from: "A716-446655440000"
        size: 100
      - from: "A716-446655441000"
        size: 200
    tags:
      - Key: Site
        Value: RCDN

- name: Delete UUID Pool
  cisco.intersight.intersight_uuid_pool:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    organization: DevNet
    name: lab-uuid-pool
    state: absent
'''

RETURN = r'''
api_response:
  description: The API response output returned by the specified resource.
  returned: always
  type: dict
  sample:
    "api_response": {
        "AccountMoid": "",
        "Ancestors": [],
        "Assigned": 0,
        "AssignmentOrder": "sequential",
        "ClassId": "uuidpool.Pool",
        "CreateTime": "",
        "Description": "UUID Pool for lab use",
        "DomainGroupMoid": "",
        "ModTime": "",
        "Moid": "",
        "Name": "lab-uuid-pool",
        "ObjectType": "uuidpool.Pool",
        "Organization": {},
        "Owners": [],
        "PermissionResources": [],
        "Prefix": "550E8400-E29B-41D4",
        "Reservations": [],
        "Reserved": 0,
        "ShadowPools": [],
        "SharedScope": "",
        "Size": 300,
        "Tags": [
            {
                "Key": "Site",
                "Value": "RCDN"
            }
        ],
        "UuidSuffixBlocks": [
            {
                "From": "A716-446655440000",
                "Size": 100
            },
            {
                "From": "A716-446655441000",
                "Size": 200
            }
        ]
    }
'''


import re
from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.intersight.plugins.module_utils.intersight import IntersightModule, intersight_argument_spec


def validate_uuid_format(uuid_string, field_sizes):
    """
    Validate UUID format with flexible field sizes
    where x represents a hexadecimal character (0-9, A-F)
    Args:
        uuid_string (str): The UUID string to validate
        field_sizes (list): List of integers representing the size of each field
                           (e.g., [4, 4, 8] for XXXX-XXXX-XXXXXXXX format)
    Returns:
        bool: True if valid, False otherwise
    """
    if not uuid_string or not field_sizes:
        return False

    # Build pattern dynamically based on field sizes
    pattern_parts = []
    for size in field_sizes:
        pattern_parts.append(f'[0-9A-F]{{{size}}}')

    pattern = f'^{"-".join(pattern_parts)}$'
    return bool(re.match(pattern, uuid_string))


def main():
    argument_spec = intersight_argument_spec.copy()
    argument_spec.update(
        state=dict(type='str', choices=['present', 'absent'], default='present'),
        organization=dict(type='str', default='default'),
        name=dict(type='str', required=True),
        description=dict(type='str', aliases=['descr']),
        tags=dict(type='list', elements='dict'),
        assignment_order=dict(type='str', choices=['default', 'sequential'], default='default'),
        prefix=dict(type='str'),
        uuid_suffix_blocks=dict(type='list', elements='dict'),
    )

    module = AnsibleModule(
        argument_spec,
        supports_check_mode=True,
        required_if=[
            ('state', 'present', ['prefix', 'uuid_suffix_blocks']),
        ],
    )

    # Validate UUID prefix format (xxxx-xxxx-xxxxxxxx)
    if module.params['prefix'] and not validate_uuid_format(module.params['prefix'], [8, 4, 4]):
        module.fail_json(msg="Invalid UUID prefix format. Expected: XXXXXXXX-XXXX-XXXX (where X is a hexadecimal character)")

    # Validate UUID suffix blocks (xxxx-xxxxxxxxxxxx)
    if module.params['uuid_suffix_blocks']:
        for uuid_block in module.params['uuid_suffix_blocks']:
            if 'from' in uuid_block and uuid_block['from'] and not validate_uuid_format(uuid_block['from'], [4, 12]):
                module.fail_json(msg="Invalid UUID suffix format in uuid_suffix_blocks. Expected: XXXX-XXXXXXXXXXXX (where X is a hexadecimal character)")

    intersight = IntersightModule(module)
    intersight.result['api_response'] = {}
    intersight.result['trace_id'] = ''

    # Resource path used to configure pool
    resource_path = '/uuidpool/Pools'
    # Define API body used in compares or create
    intersight.api_body = {
        'Organization': {
            'Name': intersight.module.params['organization'],
        },
        'Name': intersight.module.params['name'],

    }
    if module.params['state'] == 'present':
        intersight.set_tags_and_description()
        intersight.api_body['AssignmentOrder'] = intersight.module.params['assignment_order']
        intersight.api_body['Prefix'] = intersight.module.params['prefix']
        UuidSuffixBlocks = []
        for uuid_block in intersight.module.params['uuid_suffix_blocks']:
            block = {}
            block['From'] = uuid_block['from']
            block['Size'] = uuid_block['size']
            UuidSuffixBlocks.append(block)
        intersight.api_body['UuidSuffixBlocks'] = UuidSuffixBlocks

    intersight.configure_policy_or_profile(resource_path=resource_path)

    module.exit_json(**intersight.result)


if __name__ == '__main__':
    main()



#!/usr/bin/python

# Copyright (c) 2017, Ted Trask <ttrask01@yahoo.com>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: awall
short_description: Manage awall policies
author: Ted Trask (@tdtrask) <ttrask01@yahoo.com>
description:
  - This modules allows for enable/disable/activate of C(awall) policies.
  - Alpine Wall (C(awall)) generates a firewall configuration from the enabled policy files and activates the configuration
    on the system.
extends_documentation_fragment:
  - community.general._attributes
attributes:
  check_mode:
    support: full
  diff_mode:
    support: none
options:
  name:
    description:
      - One or more policy names.
    type: list
    elements: str
  state:
    description:
      - Whether the policies should be enabled or disabled.
    type: str
    choices: [disabled, enabled]
    default: enabled
  activate:
    description:
      - Activate the new firewall rules.
      - Can be run with other steps or on its own.
      - Idempotency is affected if O(activate=true), as the module always reports a changed state.
    type: bool
    default: false
notes:
  - At least one of O(name) and O(activate) is required.
"""

EXAMPLES = r"""
- name: Enable "foo" and "bar" policy
  community.general.awall:
    name: [foo bar]
    state: enabled

- name: Disable "foo" and "bar" policy and activate new rules
  community.general.awall:
    name:
      - foo
      - bar
    state: disabled
    activate: false

- name: Activate currently enabled firewall rules
  community.general.awall:
    activate: true
"""

RETURN = """ # """

import re

from ansible.module_utils.basic import AnsibleModule


def activate(module):
    cmd = f"{AWALL_PATH} activate --force"
    rc, stdout, stderr = module.run_command(cmd)
    if rc == 0:
        return True
    else:
        module.fail_json(msg="could not activate new rules", stdout=stdout, stderr=stderr)


def is_policy_enabled(module, name):
    cmd = f"{AWALL_PATH} list"
    rc, stdout, stderr = module.run_command(cmd)
    return bool(re.search(rf"^{name}\s+enabled", stdout, re.MULTILINE))


def enable_policy(module, names, act):
    policies = []
    for name in names:
        if not is_policy_enabled(module, name):
            policies.append(name)
    if not policies:
        module.exit_json(changed=False, msg="policy(ies) already enabled")
    names = " ".join(policies)
    if module.check_mode:
        cmd = f"{AWALL_PATH} list"
    else:
        cmd = f"{AWALL_PATH} enable {names}"
    rc, stdout, stderr = module.run_command(cmd)
    if rc != 0:
        module.fail_json(msg=f"failed to enable {names}", stdout=stdout, stderr=stderr)
    if act and not module.check_mode:
        activate(module)
    module.exit_json(changed=True, msg=f"enabled awall policy(ies): {names}")


def disable_policy(module, names, act):
    policies = []
    for name in names:
        if is_policy_enabled(module, name):
            policies.append(name)
    if not policies:
        module.exit_json(changed=False, msg="policy(ies) already disabled")
    names = " ".join(policies)
    if module.check_mode:
        cmd = f"{AWALL_PATH} list"
    else:
        cmd = f"{AWALL_PATH} disable {names}"
    rc, stdout, stderr = module.run_command(cmd)
    if rc != 0:
        module.fail_json(msg=f"failed to disable {names}", stdout=stdout, stderr=stderr)
    if act and not module.check_mode:
        activate(module)
    module.exit_json(changed=True, msg=f"disabled awall policy(ies): {names}")


def main():
    module = AnsibleModule(
        argument_spec=dict(
            state=dict(type="str", default="enabled", choices=["disabled", "enabled"]),
            name=dict(type="list", elements="str"),
            activate=dict(type="bool", default=False),
        ),
        required_one_of=[["name", "activate"]],
        supports_check_mode=True,
    )
    module.run_command_environ_update = {"LANGUAGE": "C", "LC_ALL": "C"}

    global AWALL_PATH
    AWALL_PATH = module.get_bin_path("awall", required=True)

    p = module.params

    if p["name"]:
        if p["state"] == "enabled":
            enable_policy(module, p["name"], p["activate"])
        elif p["state"] == "disabled":
            disable_policy(module, p["name"], p["activate"])

    if p["activate"]:
        if not module.check_mode:
            activate(module)
        module.exit_json(changed=True, msg="activated awall rules")

    module.fail_json(msg="no action defined")


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2017, Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: autoscaling_lifecycle_hook
version_added: 1.0.0
short_description: Create, delete or update AWS ASG Lifecycle Hooks
description:
  - Will create a new hook when I(state=present) and no given Hook is found.
  - Will update an existing hook when I(state=present) and a Hook is found, but current and provided parameters differ.
  - Will delete the hook when I(state=absent) and a Hook is found.
  - Prior to release 5.0.0 this module was called C(community.aws.ec2_asg_lifecycle_hook).
    The usage did not change.
author:
  - Igor 'Tsigankov' Eyrich (@tsiganenok) <tsiganenok@gmail.com>
options:
  state:
    description:
      - Create or delete Lifecycle Hook.
      - When I(state=present) updates existing hook or creates a new hook if not found.
    choices: ['present', 'absent']
    default: present
    type: str
  lifecycle_hook_name:
    description:
      - The name of the lifecycle hook.
    required: true
    type: str
  autoscaling_group_name:
    description:
      - The name of the Auto Scaling group to which you want to assign the lifecycle hook.
    required: true
    type: str
  transition:
    description:
      - The instance state to which you want to attach the lifecycle hook.
      - Required when I(state=present).
    choices: ['autoscaling:EC2_INSTANCE_TERMINATING', 'autoscaling:EC2_INSTANCE_LAUNCHING']
    type: str
  role_arn:
    description:
      - The ARN of the IAM role that allows the Auto Scaling group to publish to the specified notification target.
    type: str
  notification_target_arn:
    description:
      - The ARN of the notification target that Auto Scaling will use to notify you when an
        instance is in the transition state for the lifecycle hook.
      - This target can be either an SQS queue or an SNS topic.
      - If you specify an empty string, this overrides the current ARN.
    type: str
  notification_meta_data:
    description:
      - Contains additional information that you want to include any time Auto Scaling sends a message to the notification target.
    type: str
  heartbeat_timeout:
    description:
      - The amount of time, in seconds, that can elapse before the lifecycle hook times out.
        When the lifecycle hook times out, Auto Scaling performs the default action.
        You can prevent the lifecycle hook from timing out by calling RecordLifecycleActionHeartbeat.
      - By default Amazon AWS will use C(3600) (1 hour).
    type: int
  default_result:
    description:
      - Defines the action the Auto Scaling group should take when the lifecycle hook timeout
        elapses or if an unexpected failure occurs.
    choices: ['ABANDON', 'CONTINUE']
    default: ABANDON
    type: str
extends_documentation_fragment:
  - amazon.aws.common.modules
  - amazon.aws.region.modules
  - amazon.aws.boto3
"""

EXAMPLES = r"""
- name: Create / Update lifecycle hook
  community.aws.autoscaling_lifecycle_hook:
    region: eu-central-1
    state: present
    autoscaling_group_name: example
    lifecycle_hook_name: example
    transition: autoscaling:EC2_INSTANCE_LAUNCHING
    heartbeat_timeout: 7000
    default_result: ABANDON

- name: Delete lifecycle hook
  community.aws.autoscaling_lifecycle_hook:
    region: eu-central-1
    state: absent
    autoscaling_group_name: example
    lifecycle_hook_name: example
"""

RETURN = r"""
---
auto_scaling_group_name:
    description: The unique name of the auto scaling group.
    returned: success
    type: str
    sample: "myasg"
default_result:
    description:  Defines the action the Auto Scaling group should take when the lifecycle hook timeout elapses or if an unexpected failure occurs.
    returned: success
    type: str
    sample: CONTINUE
global_timeout:
    description: The maximum time, in seconds, that an instance can remain in a C(Pending:Wait) or C(Terminating:Wait) state.
    returned: success
    type: int
    sample: 172800
heartbeat_timeout:
    description: The maximum time, in seconds, that can elapse before the lifecycle hook times out.
    returned: success
    type: int
    sample: 3600
lifecycle_hook_name:
    description: The name of the lifecycle hook.
    returned: success
    type: str
    sample: "mylifecyclehook"
lifecycle_transition:
    description: The instance state to which lifecycle hook should be attached.
    returned: success
    type: str
    sample: "autoscaling:EC2_INSTANCE_LAUNCHING"
"""


try:
    import botocore
except ImportError:
    pass  # handled by AnsibleAWSModule

from ansible.module_utils.common.dict_transformations import camel_dict_to_snake_dict

from ansible_collections.community.aws.plugins.module_utils.modules import AnsibleCommunityAWSModule as AnsibleAWSModule


def create_lifecycle_hook(connection, module):
    lch_name = module.params.get("lifecycle_hook_name")
    asg_name = module.params.get("autoscaling_group_name")
    transition = module.params.get("transition")
    role_arn = module.params.get("role_arn")
    notification_target_arn = module.params.get("notification_target_arn")
    notification_meta_data = module.params.get("notification_meta_data")
    heartbeat_timeout = module.params.get("heartbeat_timeout")
    default_result = module.params.get("default_result")

    return_object = {}
    return_object["changed"] = False

    lch_params = {
        "LifecycleHookName": lch_name,
        "AutoScalingGroupName": asg_name,
        "LifecycleTransition": transition,
    }

    if role_arn:
        lch_params["RoleARN"] = role_arn

    if notification_target_arn:
        lch_params["NotificationTargetARN"] = notification_target_arn

    if notification_meta_data:
        lch_params["NotificationMetadata"] = notification_meta_data

    if heartbeat_timeout:
        lch_params["HeartbeatTimeout"] = heartbeat_timeout

    if default_result:
        lch_params["DefaultResult"] = default_result

    try:
        existing_hook = connection.describe_lifecycle_hooks(
            AutoScalingGroupName=asg_name,
            LifecycleHookNames=[lch_name],
        )["LifecycleHooks"]
    except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
        module.fail_json_aws(e, msg="Failed to get Lifecycle Hook")

    if not existing_hook:
        try:
            if module.check_mode:
                module.exit_json(
                    changed=True, msg="Would have created AutoScalingGroup Lifecycle Hook if not in check_mode."
                )
            return_object["changed"] = True
            connection.put_lifecycle_hook(**lch_params)
            return_object["lifecycle_hook_info"] = connection.describe_lifecycle_hooks(
                AutoScalingGroupName=asg_name, LifecycleHookNames=[lch_name]
            )["LifecycleHooks"]
        except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
            module.fail_json_aws(e, msg="Failed to create LifecycleHook")

    else:
        added, removed, modified, same = dict_compare(lch_params, existing_hook[0])
        if modified:
            try:
                if module.check_mode:
                    module.exit_json(
                        changed=True, msg="Would have modified AutoScalingGroup Lifecycle Hook if not in check_mode."
                    )
                return_object["changed"] = True
                connection.put_lifecycle_hook(**lch_params)
                return_object["lifecycle_hook_info"] = connection.describe_lifecycle_hooks(
                    AutoScalingGroupName=asg_name, LifecycleHookNames=[lch_name]
                )["LifecycleHooks"]
            except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
                module.fail_json_aws(e, msg="Failed to create LifecycleHook")

    module.exit_json(**camel_dict_to_snake_dict(return_object))


def dict_compare(d1, d2):
    d1_keys = set(d1.keys())
    d2_keys = set(d2.keys())
    intersect_keys = d1_keys.intersection(d2_keys)
    added = d1_keys - d2_keys
    removed = d2_keys - d1_keys
    modified = False
    for key in d1:
        if d1[key] != d2[key]:
            modified = True
            break

    same = set(o for o in intersect_keys if d1[o] == d2[o])
    return added, removed, modified, same


def delete_lifecycle_hook(connection, module):
    lch_name = module.params.get("lifecycle_hook_name")
    asg_name = module.params.get("autoscaling_group_name")

    return_object = {}
    return_object["changed"] = False

    try:
        all_hooks = connection.describe_lifecycle_hooks(
            AutoScalingGroupName=asg_name,
        )
    except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
        module.fail_json_aws(e, msg="Failed to get Lifecycle Hooks")

    for hook in all_hooks["LifecycleHooks"]:
        if hook["LifecycleHookName"] == lch_name:
            lch_params = {
                "LifecycleHookName": lch_name,
                "AutoScalingGroupName": asg_name,
            }

            try:
                if module.check_mode:
                    module.exit_json(
                        changed=True, msg="Would have deleted AutoScalingGroup Lifecycle Hook if not in check_mode."
                    )
                connection.delete_lifecycle_hook(**lch_params)
                return_object["changed"] = True
                return_object["lifecycle_hook_removed"] = {
                    "LifecycleHookName": lch_name,
                    "AutoScalingGroupName": asg_name,
                }
            except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
                module.fail_json_aws(e, msg="Failed to delete LifecycleHook")
        else:
            pass

    module.exit_json(**camel_dict_to_snake_dict(return_object))


def main():
    argument_spec = dict(
        autoscaling_group_name=dict(required=True, type="str"),
        lifecycle_hook_name=dict(required=True, type="str"),
        transition=dict(
            type="str", choices=["autoscaling:EC2_INSTANCE_TERMINATING", "autoscaling:EC2_INSTANCE_LAUNCHING"]
        ),
        role_arn=dict(type="str"),
        notification_target_arn=dict(type="str"),
        notification_meta_data=dict(type="str"),
        heartbeat_timeout=dict(type="int"),
        default_result=dict(default="ABANDON", choices=["ABANDON", "CONTINUE"]),
        state=dict(default="present", choices=["present", "absent"]),
    )

    module = AnsibleAWSModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
        required_if=[["state", "present", ["transition"]]],
    )

    state = module.params.get("state")

    connection = module.client("autoscaling")

    if state == "present":
        create_lifecycle_hook(connection, module)
    elif state == "absent":
        delete_lifecycle_hook(connection, module)


if __name__ == "__main__":
    main()
